@@ -32,21 +32,29 @@ email                : fh@ez.no
 void Options5::init()
 {
     m_pOSDPreview = new OSDPreviewWidget( this ); //must be child!!!
-    m_pOSDPreview->setAlignment( (OSDWidget::Alignment)AmarokConfig::osdAlignment() );
+    m_pOSDPreview->setAlignment( static_cast<OSDWidget::Alignment>( AmarokConfig::osdAlignment() ) );
     m_pOSDPreview->setOffset( AmarokConfig::osdYOffset() );
 
-    connect( m_pOSDPreview, SIGNAL(positionChanged()), SLOT(slotPositionChanged()) );
+    connect( m_pOSDPreview, SIGNAL( positionChanged() ), SLOT( slotPositionChanged() ) );
 
     const int numScreens = QApplication::desktop()->numScreens();
     for( int i = 0; i < numScreens; i++ )
         kcfg_OsdScreen->insertItem( QString::number( i ) );
 
-    connect( kcfg_OsdDrawShadow, SIGNAL(toggled(bool)), m_pOSDPreview, SLOT(setDrawShadow(bool)) );
-    connect( kcfg_OsdTextColor, SIGNAL(changed(const QColor&)), m_pOSDPreview, SLOT(setTextColor(const QColor&)) );
-    connect( kcfg_OsdBackgroundColor, SIGNAL(changed(const QColor&)), m_pOSDPreview, SLOT(setBackgroundColor(const QColor&)) );
-    connect( kcfg_OsdFont, SIGNAL(fontSelected(const QFont&)), m_pOSDPreview, SLOT(setFont(const QFont&)) );
-    connect( kcfg_OsdScreen, SIGNAL(activated(int)), m_pOSDPreview, SLOT(setScreen(int)) );
-    connect( kcfg_OsdEnabled, SIGNAL(toggled(bool)), m_pOSDPreview, SLOT(setShown(bool)) );
+    connect( kcfg_OsdDrawShadow,      SIGNAL( toggled(bool) ),
+             m_pOSDPreview,           SLOT( setDrawShadow(bool) ) );
+    connect( kcfg_OsdTextColor,       SIGNAL( changed(const QColor&) ),
+             m_pOSDPreview,           SLOT( setTextColor(const QColor&) ) );
+    connect( kcfg_OsdUseCustomColors, SIGNAL( toggled(bool) ),
+             this,                    SLOT( useCustomColorsToggled(bool) ) );
+    connect( kcfg_OsdBackgroundColor, SIGNAL( changed(const QColor&) ),
+             m_pOSDPreview,           SLOT( setBackgroundColor(const QColor&) ) );
+    connect( kcfg_OsdFont,            SIGNAL( fontSelected(const QFont&) ),
+             m_pOSDPreview,           SLOT( setFont(const QFont&) ) );
+    connect( kcfg_OsdScreen,          SIGNAL( activated(int) ),
+             m_pOSDPreview,           SLOT( setScreen(int) ) );
+    connect( kcfg_OsdEnabled,         SIGNAL( toggled(bool) ),
+             m_pOSDPreview,           SLOT( setShown(bool) ) );
 
     amaroK::QStringx text = i18n(
             "<h3>Tags Displayed in OSD</h3>"
@@ -66,6 +74,9 @@ void Options5::init()
                 "<li>Comment - %12"
                 "<li>Score - %13"
                 "<li>Playcount - %14"
+                "<li>Disc Number - %15"
+                "<li>Rating - %16"
+                "<li>Moodbar - %17"
                 "</ul>"
             "If you surround sections of text that contain a token with curly-braces, that section will be hidden if the token is empty, for example:"
                 "<pre>%11</pre>"
@@ -75,7 +86,8 @@ void Options5::init()
             // we don't translate these, it is not sensible to do so
             << "%title" << "%album"   << "%artist" << "%genre"     << "%bitrate"
             << "%year " << "%length"  << "%track"  << "%file"      << "%directory"
-            << "%type"  << "%comment" << "%score"  << "%playcount"
+            << "%type"  << "%comment" << "%score"  << "%playcount" << "%discnumber" 
+            << "%rating" << "%moodbar"
             << "%title {Score: %score}" ) );
 }
 
@@ -109,11 +121,5 @@ Options5::showEvent( QShowEvent* )
 void
 Options5::useCustomColorsToggled( bool on )
 {
-    if( on ) {
-        //use base functions so we don't call show() 3 times
-        m_pOSDPreview->OSDWidget::setTextColor( kcfg_OsdTextColor->color() );
-        m_pOSDPreview->OSDWidget::setBackgroundColor( kcfg_OsdBackgroundColor->color() );
-    }
-    else
-        m_pOSDPreview->unsetColors();
+    m_pOSDPreview->setUseCustomColors( on, kcfg_OsdTextColor->color(), kcfg_OsdBackgroundColor->color() );
 }
