@@ -5,6 +5,9 @@
 
 #include "amarok.h"
 #include "amarokconfig.h"
+#include "collectiondb.h"
+#include "config.h"        //for AMAZON_SUPPORT
+#include "covermanager.h"
 #include "coverfetcher.h"
 #include "debug.h"
 #include "statusbar.h"
@@ -19,15 +22,107 @@
 #include <kcombobox.h>
 #include <kcursor.h> //waiting cursor
 #include <kdialog.h>
+#include <kiconloader.h>
+#include <kfiledialog.h>
 #include <kio/job.h>
 #include <kio/jobclasses.h>
 #include <klineedit.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kpopupmenu.h>
 #include <kpushbutton.h>
 #include <kwin.h>
 
 
+namespace amaroK {
+    void coverContextMenu( QWidget *parent, QPoint point, const QString &artist, const QString &album, bool showCoverManager ) {
+        KPopupMenu menu;
+        enum { SHOW, FETCH, CUSTOM, DELETE, MANAGER };
+
+        menu.insertTitle( i18n( "Cover Image" ) );
+
+        menu.insertItem( SmallIconSet( amaroK::icon( "zoom" ) ), i18n( "&Show Fullsize" ), SHOW );
+        menu.insertItem( SmallIconSet( amaroK::icon( "download" ) ), i18n( "&Fetch From amazon.%1" ).arg(CoverManager::amazonTld()), FETCH );
+        menu.insertItem( SmallIconSet( amaroK::icon( "files" ) ), i18n( "Set &Custom Cover" ), CUSTOM );
+        menu.insertSeparator();
+
+        menu.insertItem( SmallIconSet( amaroK::icon( "remove" ) ), i18n("&Unset Cover"), DELETE );
+        if ( showCoverManager ) {
+            menu.insertSeparator();
+            menu.insertItem( SmallIconSet( amaroK::icon( "covermanager" ) ), i18n( "Cover Manager" ), MANAGER );
+        }
+        #ifndef AMAZON_SUPPORT
+        menu.setItemEnabled( FETCH, false );
+        #endif
+        bool disable = !CollectionDB::instance()->albumImage( artist, album, 0 ).contains( "nocover" );
+        menu.setItemEnabled( SHOW, disable );
+        menu.setItemEnabled( DELETE, disable );
+
+        switch( menu.exec( point ) )
+        {
+        case SHOW:
+            CoverManager::viewCover( artist, album, parent );
+            break;
+
+        case DELETE:
+        {
+            const int button = KMessageBox::warningContinueCancel( parent,
+                i18n( "Are you sure you want to remove this cover from the Collection?" ),
+                QString::null,
+                KStdGuiItem::del() );
+
+            if ( button == KMessageBox::Continue )
+                CollectionDB::instance()->removeAlbumImage( artist, album );
+            break;
+        }
+
+        case FETCH:
+        #ifdef AMAZON_SUPPORT
+            CollectionDB::instance()->fetchCover( parent, artist, album, false );
+            break;
+        #endif
+
+        case CUSTOM:
+        {
+            QString artist_id; artist_id.setNum( CollectionDB::instance()->artistID( artist ) );
+            QString album_id; album_id.setNum( CollectionDB::instance()->albumID( album ) );
+            QStringList values = CollectionDB::instance()->albumTracks( artist_id, album_id );
+            QString startPath = ":homedir";
+
+            if ( !values.isEmpty() ) {
+                KURL url;
+                url.setPath( values.first() );
+                startPath = url.directory();
+            }
+
+            KURL file = KFileDialog::getImageOpenURL( startPath, parent, i18n("Select Cover Image File") );
+            if ( !file.isEmpty() )
+                CollectionDB::instance()->setAlbumImage( artist, album, file );
+            break;
+        }
+
+        case MANAGER:
+            CoverManager::showOnce( album );
+            break;
+        }
+    }
+}
+
+
+
+CoverLabel::CoverLabel ( QWidget * parent, const char * name, WFlags f )
+        : QLabel( parent, name, f)
+{}
+
+
+void CoverLabel::mouseReleaseEvent(QMouseEvent *pEvent) {
+    if (pEvent->button() == LeftButton || pEvent->button() == RightButton)
+    {
+        amaroK::coverContextMenu( this, pEvent->globalPos(), m_artist, m_album, false );
+    }
+}
+
+
 CoverFetcher::CoverFetcher( QWidget *parent, const QString &artist, QString album )
         : QObject( parent, "CoverFetcher" )
         , m_artist( artist )
@@ -99,6 +194,8 @@ CoverFetcher::startFetch()
     const QString LICENSE( "D1URM11J3F2CEH" );
 
     // reset all values
+    m_coverAmazonUrls.clear();
+    m_coverAsins.clear();
     m_coverUrls.clear();
     m_coverNames.clear();
     m_xml = QString::null;
@@ -200,10 +297,14 @@ CoverFetcher::finishedXmlFetch( KIO::Job *job ) //SLOT
 
     debug() << "Fetching size: " << size << endl;
 
+    m_coverAsins.clear();
+    m_coverAmazonUrls.clear();
     m_coverUrls.clear();
     m_coverNames.clear();
     for( QDomNode node = details; !node.isNull(); node = node.nextSibling() ) {
-        QString url = node.namedItem( size ).firstChild().toText().nodeValue();
+        QString amazonUrl = node.attributes().namedItem( "url" ).toAttr().value();
+        QString coverUrl = node.namedItem( size ).firstChild().toText().nodeValue();
+        QString asin = node.namedItem( "Asin" ).firstChild().toText().nodeValue();
         QString name = node.namedItem( "ProductName" ).firstChild().toText().nodeValue();
 
     const QDomNode  artists = node.namedItem("Artists");
@@ -211,11 +312,13 @@ CoverFetcher::finishedXmlFetch( KIO::Job *job ) //SLOT
     QString artist="";
     if (!artists.isNull()) artist = artists.namedItem( "Artist" ).firstChild().toText().nodeValue();
 
-        debug() << "name:" << name << " artist:" << artist << " url:" << url << endl;
+        debug() << "name:" << name << " artist:" << artist << " url:" << coverUrl << endl;
 
-        if( !url.isEmpty() )
+        if( !coverUrl.isEmpty() )
         {
-            m_coverUrls += url;
+            m_coverAmazonUrls += amazonUrl;
+            m_coverAsins += asin;
+            m_coverUrls += coverUrl;
             m_coverNames += artist + " - " + name;
         }
     }
@@ -274,6 +377,12 @@ CoverFetcher::attemptAnotherFetch()
 
         m_currentCoverName = m_coverNames.front();
         m_coverNames.pop_front();
+
+        m_amazonURL = m_coverAmazonUrls.front();
+        m_coverAmazonUrls.pop_front();
+
+        m_asin = m_coverAsins.front();
+        m_coverAsins.pop_front();
     }
 
     else if( !m_xml.isEmpty() && m_size > 0 ) {
@@ -293,6 +402,10 @@ CoverFetcher::attemptAnotherFetch()
         // we have exhausted all the predetermined queries
         // so lets let the user give it a try
         getUserQuery( i18n("You have seen all the covers Amazon returned using the query below. Perhaps you can refine it:") );
+        m_coverAmazonUrls.clear();
+        m_coverAsins.clear();
+        m_coverUrls.clear();
+        m_coverNames.clear();
     }
     else
         finishWithError( i18n("No cover found") );
@@ -304,24 +417,45 @@ CoverFetcher::attemptAnotherFetch()
     class EditSearchDialog : public KDialog
     {
     public:
-        EditSearchDialog( QWidget* parent, const QString &text, const QString &keyword )
+        EditSearchDialog( QWidget* parent, const QString &text, const QString &keyword, CoverFetcher *fetcher )
                 : KDialog( parent )
         {
             setCaption( i18n( "Amazon Query Editor" ) );
 
-            QVBoxLayout *vbox = new QVBoxLayout( this, 8, 8 );
-            QHBoxLayout *hbox = new QHBoxLayout( 8 );
+            // amazon combo box
+            KComboBox* amazonLocale = new KComboBox( this );
+            amazonLocale->insertItem( i18n("International"), CoverFetcher::International );
+            amazonLocale->insertItem( i18n("Canada"), CoverFetcher::Canada );
+            amazonLocale->insertItem( i18n("France"), CoverFetcher::France );
+            amazonLocale->insertItem( i18n("Germany"), CoverFetcher::Germany );
+            amazonLocale->insertItem( i18n("Japan"), CoverFetcher::Japan);
+            amazonLocale->insertItem( i18n("United Kingdom"), CoverFetcher::UK );
+            if( CoverManager::instance() )
+                connect( amazonLocale, SIGNAL( activated(int) ),
+                        CoverManager::instance(), SLOT( changeLocale(int) ) );
+            else
+                connect( amazonLocale, SIGNAL( activated(int) ),
+                        fetcher, SLOT( changeLocale(int) ) );
+            QHBoxLayout *hbox1 = new QHBoxLayout( 8 );
+            hbox1->addWidget( new QLabel( i18n( "Amazon Locale: " ), this ) );
+            hbox1->addWidget( amazonLocale );
+
+            int currentLocale = CoverFetcher::localeStringToID( AmarokConfig::amazonLocale() );
+            amazonLocale->setCurrentItem( currentLocale );
 
             KPushButton* cancelButton = new KPushButton( KStdGuiItem::cancel(), this );
             KPushButton* searchButton = new KPushButton( i18n("&Search"), this );
 
-            hbox->addItem( new QSpacerItem( 80, 8, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
-            hbox->addWidget( searchButton );
-            hbox->addWidget( cancelButton );
+            QHBoxLayout *hbox2 = new QHBoxLayout( 8 );
+            hbox2->addItem( new QSpacerItem( 160, 8, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
+            hbox2->addWidget( searchButton );
+            hbox2->addWidget( cancelButton );
 
+            QVBoxLayout *vbox = new QVBoxLayout( this, 8, 8 );
+            vbox->addLayout( hbox1 );
             vbox->addWidget( new QLabel( "<qt>" + text, this ) );
             vbox->addWidget( new KLineEdit( keyword, this, "Query" ) );
-            vbox->addLayout( hbox );
+            vbox->addLayout( hbox2 );
 
             searchButton->setDefault( true );
 
@@ -335,6 +469,48 @@ CoverFetcher::attemptAnotherFetch()
         QString query() { return static_cast<KLineEdit*>(child( "Query" ))->text(); }
     };
 
+QString
+CoverFetcher::localeIDToString( int id )//static
+{
+    switch ( id )
+    {
+    case International:
+        return "us";
+    case Canada:
+        return "ca";
+    case France:
+        return "fr";
+    case Germany:
+        return "de";
+    case Japan:
+        return "jp";
+    case UK:
+        return "uk";
+    }
+
+    return "us";
+}
+
+int
+CoverFetcher::localeStringToID( const QString &s )
+{
+    int id = International;
+    if( s == "fr" ) id = France;
+    else if( s == "de" ) id = Germany;
+    else if( s == "jp" ) id = Japan;
+    else if( s == "uk" ) id = UK;
+    else if( s == "ca" ) id = Canada;
+
+    return id;
+}
+
+void
+CoverFetcher::changeLocale( int id )//SLOT
+{
+    QString locale = localeIDToString( id );
+    AmarokConfig::setAmazonLocale( locale );
+}
+
 
 void
 CoverFetcher::getUserQuery( QString explanation )
@@ -343,9 +519,10 @@ CoverFetcher::getUserQuery( QString explanation )
         explanation = i18n("Ask Amazon for covers using this query:");
 
     EditSearchDialog dialog(
-            (QWidget*)parent(),
+            static_cast<QWidget*>( parent() ),
             explanation,
-            m_userQuery );
+            m_userQuery,
+            this );
 
     switch( dialog.exec() )
     {
@@ -376,8 +553,8 @@ CoverFetcher::getUserQuery( QString explanation )
             QLabel      *labelName = new QLabel( this );
             QHBox       *buttons   = new QHBox( this );
             KPushButton *save      = new KPushButton( KStdGuiItem::save(), buttons );
-            KPushButton *newsearch = new KPushButton( i18n( "New &Search..." ), buttons, "NewSearch" );
-            KPushButton *nextcover = new KPushButton( i18n( "Next &Cover" ), buttons, "NextCover" );
+            KPushButton *newsearch = new KPushButton( i18n( "Ne&w Search..." ), buttons, "NewSearch" );
+            KPushButton *nextcover = new KPushButton( i18n( "&Next Cover" ), buttons, "NextCover" );
             KPushButton *cancel    = new KPushButton( KStdGuiItem::cancel(), buttons );
 
             labelPix ->setAlignment( Qt::AlignHCenter );
@@ -410,7 +587,7 @@ CoverFetcher::getUserQuery( QString explanation )
 void
 CoverFetcher::showCover()
 {
-    CoverFoundDialog dialog( (QWidget*)parent(), m_image, m_currentCoverName );
+    CoverFoundDialog dialog( static_cast<QWidget*>( parent() ), m_image, m_currentCoverName );
 
     switch( dialog.exec() )
     {
@@ -419,6 +596,10 @@ CoverFetcher::showCover()
         break;
     case 1000: //showQueryEditor()
         getUserQuery();
+        m_coverAmazonUrls.clear();
+        m_coverAsins.clear();
+        m_coverUrls.clear();
+        m_coverNames.clear();
         break;
     case 1001: //nextCover()
         attemptAnotherFetch();
