@@ -112,8 +112,8 @@ HelixSoundDevice::HelixSoundDevice( QWidget *parent,
 {
    QGridLayout *grid = (QGridLayout*)parent->layout();
 
-   deviceComboBox = new KComboBox( FALSE, parent, "deviceComboBox" );
-   deviceComboBox->insertItem("oss");  // I believe these are not subject to translation (they dont seem to be in xine, 
+   deviceComboBox = new KComboBox( false, parent, "deviceComboBox" );
+   deviceComboBox->insertItem("oss");  // I believe these are not subject to translation (they don't seem to be in xine, 
 #ifdef USE_HELIX_ALSA
    deviceComboBox->insertItem("alsa"); // and neither are the equivalents in gst (osssink and alsasink)
 #endif
@@ -137,6 +137,8 @@ HelixSoundDevice::HelixSoundDevice( QWidget *parent,
     connect( (QWidget *) lineEdit_outputDevice, SIGNAL( textChanged( const QString& )), pluginConfig, SIGNAL(viewChanged()) );
     connect( checkBox_outputDevice, SIGNAL( toggled(bool) ), lineEdit_outputDevice, SLOT( setEnabled(bool) ) );
     connect( checkBox_outputDevice, SIGNAL( toggled(bool) ), pluginConfig, SIGNAL(viewChanged()) );
+
+    connect( checkBox_outputDevice, SIGNAL( toggled(bool) ), this, SLOT( slotDeviceChecked(bool) ) );
     grid->addWidget( (QWidget *) lineEdit_outputDevice, row, 1 );
 
     if (HelixConfig::deviceenabled())
@@ -160,7 +162,6 @@ HelixSoundDevice::HelixSoundDevice( QWidget *parent,
 void 
 HelixSoundDevice::slotNewDevice( const QString &dev )
 {
-   debug() << "SELECT STRING WAS: " << dev.utf8() << endl;
    if (dev == "oss")
    {
       checkBox_outputDevice->setEnabled( false );
@@ -184,6 +185,17 @@ HelixSoundDevice::slotStringChanged( const QString& )
    m_changed = true;
 }
 
+void 
+HelixSoundDevice::slotDeviceChecked( bool checked )
+{
+   checkBox_outputDevice->setChecked( checked );
+   if (checked)
+      lineEdit_outputDevice->setEnabled( true );      
+   else
+      lineEdit_outputDevice->setEnabled( false );
+   m_changed = true;
+}
+
 bool 
 HelixSoundDevice::save()
 {
@@ -208,7 +220,6 @@ HelixSoundDevice::save()
 
 void HelixSoundDevice::setSoundSystem( int api )
 {
-   debug() << "SETTING SOUND SYSTEM to " << api << endl;
    switch (api)
    {
       case HelixSimplePlayer::OSS:
@@ -345,16 +356,17 @@ HelixConfigDialogBase::HelixConfigDialogBase( HelixEngine *engine, amaroK::Plugi
     le->setWordWrap(QTextEdit::NoWrap);
 
     int n = engine->numPlugins();
-    debug() << "NPLUGINS: " << n << "\n";
     const char *description, *copyright, *moreinfourl;
     row = 0;
     for (int i=0; i<n; i++)
     {
-       engine->getPluginInfo(i, description, copyright, moreinfourl);
-       le->append(QString(description));
-       le->append(QString(copyright));
-       le->append(QString(moreinfourl));
-       le->append(QString(" "));
+       if (!engine->getPluginInfo(i, description, copyright, moreinfourl))
+       {
+          le->append(QString(description));
+          le->append(QString(copyright));
+          le->append(QString(moreinfourl));
+          le->append(QString(" "));
+       }
     }
 
     le->setReadOnly(true);
@@ -392,8 +404,6 @@ HelixConfigDialogBase::save()
 {
    bool writeIt = false;
 
-   debug() << "SAVING in HelixConfigDialog\n";
-
    if (m_core->isChanged())
    {
       m_engine->m_coredir = m_core->stringValue();
@@ -422,8 +432,6 @@ HelixConfigDialogBase::save()
    {
       if( entry->isChanged() )
       {
-         debug() << "Apply: " << entry->key() << "\n";
-
          entry->setUnchanged();
       }
    }
@@ -452,7 +460,6 @@ HelixConfigDialog::HelixConfigDialog( HelixEngine *engine, QWidget *p ) : amaroK
 
 HelixConfigDialog::~HelixConfigDialog()
 {
-   debug() << "DELETING HelixConfigDialog\n";
    delete instance;
    instance = 0;
 }
