@@ -29,19 +29,22 @@
 #endif
 #include "hsphook.h"
 #include "iir_cf.h"         // IIR filter coefficients
-#include "print.h"
+#include "hspalsadevice.h"
 
-#define SCOPE_BUF_PER_BLOCK 8
 #define SCOPESIZE 512
 
-HSPPreMixAudioHook::HSPPreMixAudioHook(HelixSimplePlayer *player, int playerIndex, IHXAudioStream *pAudioStream) :
-   m_Player(player), m_lRefCount(0), m_index(playerIndex), m_stream(pAudioStream), m_count(0)
+HSPPreMixAudioHook::HSPPreMixAudioHook(HelixSimplePlayer *player, int playerIndex, IHXAudioStream *pAudioStream,
+                                       bool fadein, unsigned long fadelength) :
+   m_Player(player), m_lRefCount(0), m_index(playerIndex), m_stream(pAudioStream), m_count(0),
+   m_gaintool(0), m_gaindb(0), m_fadein(fadein), m_fadeout(false), m_fadelength(fadelength)
 {
    AddRef();
 }
 
 HSPPreMixAudioHook::~HSPPreMixAudioHook()
 {
+   if (m_gaintool)
+      gainFree(m_gaintool);
 }
 
 STDMETHODIMP
@@ -81,46 +84,99 @@ HSPPreMixAudioHook::Release()
     return 0;
 }
 
-STDMETHODIMP HSPPreMixAudioHook::OnBuffer(HXAudioData */*pAudioInData*/, HXAudioData */*pAudioOutData*/)
+int HSPPreMixAudioHook::volumeize(unsigned char *data, unsigned char *outbuf, size_t len)
+{
+   gainFeed(data, outbuf, len, m_gaintool);
+
+   return len;
+}
+
+void HSPPreMixAudioHook::setFadeout(bool fadeout)
+{ 
+   m_fadeout = fadeout; 
+   if (m_fadeout)
+   {
+      // the "time constant" (ms) is the time it takes to reach +6db of the target
+      gainSetTimeConstant((float) m_fadelength / 2.0, m_gaintool);
+      gainSetSmoothdB(FADE_MIN_dB, m_gaintool);
+   }
+}
+
+STDMETHODIMP HSPPreMixAudioHook::OnBuffer(HXAudioData *pAudioInData, HXAudioData *pAudioOutData)
 {
    m_count++;
 
 #ifdef DEBUG_PURPOSES_ONLY
    if (!(m_count % 100))
    {
-      m_Player->STDERR("PRE: time: %d  ", pAudioInData->ulAudioTime);
+      m_Player->print2stderr("PRE: time: %d  ", pAudioInData->ulAudioTime);
       switch (pAudioInData->uAudioStreamType)
       {
          case INSTANTANEOUS_AUDIO:
-            m_Player->STDERR(" INSTANTANEOUS_AUDIO ");
+            m_Player->print2stderr(" INSTANTANEOUS_AUDIO ");
             break;
          case STREAMING_AUDIO:
-            m_Player->STDERR(" STREAMING_AUDIO ");
+            m_Player->print2stderr(" STREAMING_AUDIO ");
             break;
          case TIMED_AUDIO:
-            m_Player->STDERR(" TIMED_AUDIO ");
+            m_Player->print2stderr(" TIMED_AUDIO ");
             break;
          case STREAMING_INSTANTANEOUS_AUDIO:
-            m_Player->STDERR(" STREAMING_INSTANTANEOUS_AUDIO ");
+            m_Player->print2stderr(" STREAMING_INSTANTANEOUS_AUDIO ");
             break;
       }
-      m_Player->STDERR("pAudioOutData %lx, data %lx\n", pAudioOutData, pAudioOutData->pData);
+      m_Player->print2stderr("pAudioOutData %lx, data %lx\n", pAudioOutData, pAudioOutData->pData);
    }
 #endif
 
+   unsigned char *outbuf;
+   IHXBuffer *ibuf;
+   unsigned long len;
+   unsigned char *data;
+
+   pAudioInData->pData->Get(data, len);
+
+   // provide a little margin to prevent a slight but noticeable jump in vol when the fadein ends
+   if ((m_fadein && pAudioInData->ulAudioTime < 2*m_fadelength) || m_fadeout)
+   {
+      m_Player->pCommonClassFactory->CreateInstance(CLSID_IHXBuffer, (void **) &ibuf);
+      if (ibuf)
+      {
+         ibuf->SetSize(len);
+         outbuf = ibuf->GetBuffer();
+         
+         len = volumeize(data, outbuf, len);
+
+         pAudioOutData->pData = ibuf;
+         pAudioOutData->ulAudioTime = pAudioInData->ulAudioTime;
+         pAudioOutData->uAudioStreamType = pAudioInData->uAudioStreamType;
+      }
+   }
 
    return 0;
 }
 
 STDMETHODIMP HSPPreMixAudioHook::OnInit(HXAudioFormat *pFormat)
 {
-   m_Player->STDERR("PRE MIX HOOK OnInit AudioFormat: ch %d, bps %d, sps %d, mbs %d\n", pFormat->uChannels,
+   m_Player->print2stderr("PRE MIX HOOK OnInit AudioFormat: ch %d, bps %d, sps %ld, mbs %d\n", pFormat->uChannels,
           pFormat->uBitsPerSample,
           pFormat->ulSamplesPerSec,
           pFormat->uMaxBlockSize);
 
    m_format = *pFormat;
 
+   int bps = pFormat->uBitsPerSample / 8;
+   m_gaintool = gainInit(pFormat->ulSamplesPerSec, pFormat->uChannels, bps);
+   gainSetImmediatedB(0, m_gaintool);
+
+   if (m_fadein)
+   {
+      gainSetImmediatedB(FADE_MIN_dB, m_gaintool);
+      // the "time constant" (ms) is the time it takes to reach -6db of the target
+      gainSetTimeConstant((float) m_fadelength / 2.0, m_gaintool);
+      gainSetSmoothdB(0, m_gaintool);
+   }
+
    return 0;
 }
 
@@ -198,26 +254,26 @@ STDMETHODIMP HSPPostProcessor::OnBuffer(HXAudioData *pAudioInData, HXAudioData *
 #ifdef DEBUG_PURPOSES_ONLY
    if (!(m_count % 100))
    {
-      m_Player->STDERR("POST: time: %d  ", pAudioInData->ulAudioTime);
+      m_Player->print2stderr("POST: time: %d  ", pAudioInData->ulAudioTime);
       switch (pAudioInData->uAudioStreamType)
       {
          case INSTANTANEOUS_AUDIO:
-            m_Player->STDERR(" INSTANTANEOUS_AUDIO ");
+            m_Player->print2stderr(" INSTANTANEOUS_AUDIO ");
             break;
          case STREAMING_AUDIO:
-            m_Player->STDERR(" STREAMING_AUDIO ");
+            m_Player->print2stderr(" STREAMING_AUDIO ");
             break;
          case TIMED_AUDIO:
-            m_Player->STDERR(" TIMED_AUDIO ");
+            m_Player->print2stderr(" TIMED_AUDIO ");
             break;
          case STREAMING_INSTANTANEOUS_AUDIO:
-            m_Player->STDERR(" STREAMING_INSTANTANEOUS_AUDIO ");
+            m_Player->print2stderr(" STREAMING_INSTANTANEOUS_AUDIO ");
             break;
       }
-      m_Player->STDERR("len %d\n", len);
-      m_Player->STDERR("pAudioOutData %lx, data %lx\n", pAudioOutData, pAudioOutData->pData);
+      m_Player->print2stderr("len %d\n", len);
+      m_Player->print2stderr("pAudioOutData %lx, data %lx\n", pAudioOutData, pAudioOutData->pData);
 
-      m_Player->STDERR("Volume is %d\n",m_Player->getDirectHWVolume());
+      m_Player->print2stderr("Volume is %d\n",m_Player->getDirectHWVolume());
    }
 
 #endif
@@ -277,6 +333,7 @@ STDMETHODIMP HSPPostProcessor::OnInit(HXAudioFormat *pFormat)
 {
    m_format = *pFormat;
    m_count = 0;
+   m_prevtime = 0;
 
    // set the filter coefficients, in case we need to use the equalizer
    switch(pFormat->ulSamplesPerSec)
@@ -345,7 +402,7 @@ void HSPPostProcessor::setGain(int volume)
       else
       {
          //m_gaindB = GAIN_MIN_dB + (GAIN_MAX_dB - GAIN_MIN_dB) * (float) volume / 100.0;
-         //m_Player->STDERR("GAIN set to %f\n", m_gaindB);
+         //m_Player->print2stderr("GAIN set to %f\n", m_gaindB);
          //gainSetImmediatedB(m_gaindB, m_gaintool);
 
          gainSetImmediate( (float) volume / 100.0, m_gaintool );
@@ -360,7 +417,7 @@ void HSPPostProcessor::scopeify(unsigned long time, unsigned char *data, size_t
    int bytes_per_sample = m_format.uBitsPerSample / 8;
 
    // TODO: 32 bit samples
-   if (bytes_per_sample != 1 && bytes_per_sample != 2)
+   if ( (bytes_per_sample != 1 && bytes_per_sample != 2) )
       return; // no scope
 
    unsigned long scopebuf_timeinc = (unsigned long)(1000.0 * (double)len / ((double)m_format.ulSamplesPerSec * (double)bytes_per_sample));
@@ -369,12 +426,15 @@ void HSPPostProcessor::scopeify(unsigned long time, unsigned char *data, size_t
    item->len = len;
    item->time = time;
    item->etime = time + scopebuf_timeinc;
+
+   m_prevtime = item->etime;
+
    item->nchan = m_format.uChannels;
    item->bps = bytes_per_sample;
    item->spb = len / item->nchan;
    item->spb /= bytes_per_sample;
    item->tps = (double) scopebuf_timeinc / (double) item->spb;
-   m_Player->addScopeBuf(item);
+   m_Player->addScopeBuf(item, m_index);
 }
 
 
@@ -499,7 +559,7 @@ HSPPostMixAudioHook::HSPPostMixAudioHook(HelixSimplePlayer *player, int playerIn
 {
    AddRef();
 
-   m_Player->STDERR("POST MIX HOOK CTOR\n");
+   m_Player->print2stderr("POST MIX HOOK CTOR\n");
 
    m_processor = new HSPPostProcessor(player, playerIndex);
 }
@@ -547,6 +607,7 @@ HSPPostMixAudioHook::Release()
         return m_lRefCount;
     }
 
+    m_Player->print2stderr("DELETING POST MIX HOOK index %d\n", m_index);
     delete this;
     return 0;
 }
@@ -561,17 +622,18 @@ STDMETHODIMP HSPPostMixAudioHook::OnBuffer(HXAudioData *pAudioInData, HXAudioDat
    // feed the visualizations, if this is a local file (otherwise do it in the FinalHook)
    if (m_Player->isLocal(m_index))
       m_processor->scopeify(pAudioInData->ulAudioTime, data, len);
-
+   
    return 0;
 }
 
 STDMETHODIMP HSPPostMixAudioHook::OnInit(HXAudioFormat *pFormat)
 {
-   m_Player->STDERR("POST MIX HOOK OnInit AudioFormat: idx %d ch %d, bps %d, sps %d, mbs %d\n", m_index, pFormat->uChannels,
+   m_Player->print2stderr("POST MIX HOOK OnInit AudioFormat: idx %d ch %d, bps %d, sps %ld, mbs %d\n", m_index, pFormat->uChannels,
           pFormat->uBitsPerSample,
           pFormat->ulSamplesPerSec,
           pFormat->uMaxBlockSize);
 
+
    return (m_processor->OnInit(pFormat));
 }
 
@@ -656,6 +718,8 @@ STDMETHODIMP HSPFinalAudioHook::OnBuffer(HXAudioData *pAudioInData, HXAudioData
    int i = 0;
    while (i< m_Player->numPlayers())
    {
+      if (m_Player->isPlaying(i))
+         m_processor->setIndex(i); // put the buffers on the queue of the player that's playing
       if (anyLocal = m_Player->isLocal(i))
          break;
       i++;
@@ -668,7 +732,7 @@ STDMETHODIMP HSPFinalAudioHook::OnBuffer(HXAudioData *pAudioInData, HXAudioData
 
 STDMETHODIMP HSPFinalAudioHook::OnInit(HXAudioFormat *pFormat)
 {
-   m_Player->STDERR("FINAL HOOK OnInit AudioFormat: ch %d, bps %d, sps %d, mbs %d\n",  pFormat->uChannels,
+   m_Player->print2stderr("FINAL HOOK OnInit AudioFormat: ch %d, bps %d, sps %ld, mbs %d\n",  pFormat->uChannels,
           pFormat->uBitsPerSample,
           pFormat->ulSamplesPerSec,
           pFormat->uMaxBlockSize);
