@@ -1,5 +1,8 @@
 /***************************************************************************
- *   Copyright (C) 2004,5 Max Howell <max.howell@methylblue.com>           *
+ *   Copyright (C) 2005   Christophe Thommeret <hftom@free.fr>             *
+ *             (C) 2005   Ian Monroe <ian@monroe.nu>                       *
+ *             (C) 2005,6 Mark Kretschmann <markey@web.de>                 *
+ *             (C) 2004,5 Max Howell <max.howell@methylblue.com>           *
  *             (C) 2003,4 J. Kofler <kaffeine@gmx.net>                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -14,6 +17,11 @@
 #include "xine-config.h"
 #include "xinecfg.h"
 #include "xine-engine.h"
+#include "amarok.h"
+#include "amarokconfig.h"
+//these files are from libamarok
+#include "playlist.h"
+#include "enginecontroller.h"
 
 AMAROK_EXPORT_PLUGIN( XineEngine )
 
@@ -27,6 +35,7 @@ AMAROK_EXPORT_PLUGIN( XineEngine )
 
 #include <qapplication.h>
 #include <qdir.h>
+#include <qtimer.h>
 
 extern "C"
 {
@@ -66,13 +75,15 @@ XineEngine::XineEngine()
         , m_post( 0 )
         , m_preamp( 1.0 )
         , m_stopFader( false )
+        , m_equalizerEnabled( false )
 {
-    addPluginProperty( "StreamingMode", "NoStreaming" );
     addPluginProperty( "HasConfigure", "true" );
     addPluginProperty( "HasEqualizer", "true" );
     #ifndef __NetBSD__  // NetBSD does not offer audio mixing
     addPluginProperty( "HasCrossfade", "true" );
     #endif
+    addPluginProperty("HasCDDA", "true"); // new property
+    debug() << "hello" << endl;
 }
 
 XineEngine::~XineEngine()
@@ -85,7 +96,7 @@ XineEngine::~XineEngine()
     }
 
     // NOTE The fadeout gets stuck when the EQ is active, so we skip it then
-    if( !m_equalizerEnabled && m_stream && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
+    if( !m_equalizerEnabled && m_stream && state() == Engine::Playing )
     {
         const int volume = xine_get_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL );
         const double D = 300000 * std::pow( (double)volume, -0.4951 );
@@ -128,7 +139,7 @@ XineEngine::init()
    m_xine = xine_new();
 
    if( !m_xine ) {
-      KMessageBox::error( 0, i18n("amaroK could not initialize xine.") );
+      KMessageBox::error( 0, i18n("Amarok could not initialize xine.") );
       return false;
    }
 
@@ -141,8 +152,7 @@ XineEngine::init()
 
    xine_init( m_xine );
 
-   if( !makeNewStream() )
-      return false;
+   makeNewStream();
 
    #ifndef XINE_SAFE_MODE
    startTimer( 200 ); //prunes the scope
@@ -154,28 +164,21 @@ XineEngine::init()
 bool
 XineEngine::makeNewStream()
 {
-    m_currentAudioPlugin = XineCfg::outputPlugin();
-   {
-      ///this block is so we don't crash if we fail to allocate a thingy
-      xine_stream_t     *stream;
-      xine_audio_port_t *port;
+   m_currentAudioPlugin = XineCfg::outputPlugin();
 
-      port = xine_open_audio_driver( m_xine, XineCfg::outputPlugin().local8Bit(), NULL );
-      if( !port ) {
-         //TODO make engine method that is the same but parents the dialog for us
-         KMessageBox::error( 0, i18n("xine was unable to initialize any audio-drivers.") );
-         return false;
-      }
+   m_audioPort = xine_open_audio_driver( m_xine, XineCfg::outputPlugin().local8Bit(), NULL );
+   if( !m_audioPort ) {
+      //TODO make engine method that is the same but parents the dialog for us
+      KMessageBox::error( 0, i18n("xine was unable to initialize any audio drivers.") );
+      return false;
+   }
 
-      stream = xine_stream_new( m_xine, port, NULL );
-      if( !stream ) {
-         KMessageBox::error( 0, i18n("amaroK could not create a new xine-stream.") );
-         return false;
-      }
-
-      //assign temporaries to members, the important bits are now created;
-      m_stream = stream;
-      m_audioPort = port;
+   m_stream = xine_stream_new( m_xine, m_audioPort, NULL );
+   if( !m_stream ) {
+      xine_close_audio_driver( m_xine, m_audioPort );
+      m_audioPort = NULL;
+      KMessageBox::error( 0, i18n("Amarok could not create a new xine stream.") );
+      return false;
    }
 
    if( m_eventQueue )
@@ -193,6 +196,22 @@ XineEngine::makeNewStream()
    xine_set_param( m_stream, XINE_PARAM_METRONOM_PREBUFFER, 6000 );
    xine_set_param( m_stream, XINE_PARAM_IGNORE_VIDEO, 1 );
    #endif
+#ifdef XINE_PARAM_EARLY_FINISHED_EVENT
+    if ( xine_check_version(1,1,1) && !(m_xfadeLength > 0) ) {
+        // enable gapless playback
+        debug() << "gapless playback enabled." << endl;
+        xine_set_param(m_stream, XINE_PARAM_EARLY_FINISHED_EVENT, 1 );
+    }
+#endif
+   return true;
+}
+
+// Makes sure an audio port and a stream exist.
+bool
+XineEngine::ensureStream()
+{
+   if( !m_stream )
+      return makeNewStream();
 
    return true;
 }
@@ -200,11 +219,16 @@ XineEngine::makeNewStream()
 bool
 XineEngine::load( const KURL &url, bool isStream )
 {
+    DEBUG_BLOCK
+
+    if( !ensureStream() )
+        return false;
+
     Engine::Base::load( url, isStream );
 
     if( m_xfadeLength > 0 && xine_get_status( m_stream ) == XINE_STATUS_PLAY )
     {
-       s_fader = new Fader( this );
+       s_fader = new Fader( this, m_xfadeLength );
        setEqualizerParameters( m_intPreamp, m_equalizerGains );
     }
 
@@ -212,8 +236,12 @@ XineEngine::load( const KURL &url, bool isStream )
    // why doesn't xine do this? I cannot say.
    xine_close( m_stream );
 
+   debug() << "Before xine_open() *****" << endl;
+
    if( xine_open( m_stream, QFile::encodeName( url.url() ) ) )
    {
+      debug() << "After xine_open() *****" << endl;
+
       #ifndef XINE_SAFE_MODE
       //we must ensure the scope is pruned of old buffers
       timerEvent( 0 );
@@ -225,8 +253,17 @@ XineEngine::load( const KURL &url, bool isStream )
 
       return true;
    }
+   else
+   {
+      #ifdef XINE_PARAM_GAPLESS_SWITCH
+        if ( xine_check_version(1,1,1) && !(m_xfadeLength > 0) )
+            xine_set_param( m_stream, XINE_PARAM_GAPLESS_SWITCH, 0);
+      #endif
+   }
 
+   // FAILURE to load!
    //s_fader will delete itself
+   determineAndShowErrorMessage();
 
    return false;
 }
@@ -234,23 +271,21 @@ XineEngine::load( const KURL &url, bool isStream )
 bool
 XineEngine::play( uint offset )
 {
-    if( xine_play( m_stream, 0, offset ) )
+    DEBUG_BLOCK
+
+    if( !ensureStream() )
+        return false;
+
+    const bool has_audio     = xine_get_stream_info( m_stream, XINE_STREAM_INFO_HAS_AUDIO );
+    const bool audio_handled = xine_get_stream_info( m_stream, XINE_STREAM_INFO_AUDIO_HANDLED );
+
+    if (has_audio && audio_handled && xine_play( m_stream, 0, offset ))
     {
         if( s_fader )
            s_fader->start( QThread::LowestPriority );
 
         emit stateChanged( Engine::Playing );
 
-        //HACK Emit two dummy bundles, because the context-browser expects them for
-        //     the stream metadata display.
-        if( isStream() ) {
-            Engine::SimpleMetaBundle bundle;
-            bundle.title = "dummy";
-            emit metaData( bundle );
-            bundle.title = i18n( "Radio Stream" );
-            emit metaData( bundle );
-        }
-
         return true;
     }
 
@@ -259,35 +294,80 @@ XineEngine::play( uint offset )
 
     emit stateChanged( Engine::Empty );
 
-    switch( xine_get_error( m_stream ) )
-    {
-    case XINE_ERROR_NO_INPUT_PLUGIN:
-        KMessageBox::error( 0, i18n("No input plugin available; check your installation.") );
-        break;
-    case XINE_ERROR_NO_DEMUX_PLUGIN:
-        KMessageBox::error( 0, i18n("No demux plugin available; check your installation.") );
-        break;
-    case XINE_ERROR_DEMUX_FAILED:
-        KMessageBox::error( 0, i18n("Demuxing failed; check your installation.") );
-        break;
-    default:
-        KMessageBox::error( 0, i18n("Internal error; check your installation.") );
-        break;
-    }
+    determineAndShowErrorMessage();
 
     xine_close( m_stream );
 
     return false;
 }
 
+#include "statusbar/statusbar.h"
+
+void
+XineEngine::determineAndShowErrorMessage()
+{
+    DEBUG_BLOCK
+
+    QString body;
+
+    debug() << "xine_get_error()\n";
+    switch (xine_get_error( m_stream )) {
+        case XINE_ERROR_NO_INPUT_PLUGIN:
+            body = i18n("No suitable input plugin. This often means that the url's protocol is not supported. Network failures are other possible causes.");
+        break;
+
+        case XINE_ERROR_NO_DEMUX_PLUGIN:
+            body = i18n("No suitable demux plugin. This often means that the file format is not supported.");
+        break;
+
+        case XINE_ERROR_DEMUX_FAILED:
+            body = i18n("Demuxing failed.");
+        break;
+
+        case XINE_ERROR_INPUT_FAILED:
+            body = i18n("Could not open file.");
+        break;
+
+        case XINE_ERROR_MALFORMED_MRL:
+            body = i18n("The location is malformed.");
+        break;
+
+        case XINE_ERROR_NONE:
+            // xine is thick. xine doesn't think there is an error
+            // but there may be! We check for other errors below.
+
+        default:
+            if (!xine_get_stream_info( m_stream, XINE_STREAM_INFO_AUDIO_HANDLED ))
+            {
+                // xine can read the plugin but it didn't find any codec
+                // THUS xine=daft for telling us it could handle the format in canDecode!
+                body = i18n("There is no available decoder.");
+                QString const ext = amaroK::extension( m_url.url() ).lower();
+                if (ext == "mp3" && EngineController::installDistroCodec( "xine-engine" ))
+                    return;
+            }
+            else if (!xine_get_stream_info( m_stream, XINE_STREAM_INFO_HAS_AUDIO ))
+                body = i18n("There is no audio channel!");
+        break;
+    }
+
+    amaroK::StatusBar::instance()->longMessage(
+            "<b>" + i18n("Error Loading Media") + "</b><p>" + body + "<p>" + m_url.prettyURL(),
+            KDE::StatusBar::Error );
+}
+
 void
 XineEngine::stop()
 {
+    if ( !m_stream )
+       return;
+
     m_url = KURL(); //to ensure we return Empty from state()
 
     std::fill( m_scope.begin(), m_scope.end(), 0 );
 
     xine_stop( m_stream );
+    xine_close( m_stream );
     xine_set_param( m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
 
     emit stateChanged( Engine::Empty );
@@ -312,6 +392,9 @@ XineEngine::pause()
 Engine::State
 XineEngine::state() const
 {
+    if ( !m_stream )
+       return Engine::Empty;
+
     switch( xine_get_status( m_stream ) )
     {
     case XINE_STATUS_PLAY: return xine_get_param( m_stream, XINE_PARAM_SPEED ) ? Engine::Playing : Engine::Paused;
@@ -324,33 +407,62 @@ XineEngine::state() const
 uint
 XineEngine::position() const
 {
+    if ( !m_stream )
+       return 0;
+
     int pos;
     int time = 0;
     int length;
 
     xine_get_pos_length( m_stream, &pos, &time, &length );
 
+
+    // Here we check for new metadata periodically, because xine does not emit an event
+    // in all cases (e.g. with ogg streams). See BUG 122505
+    if ( state() != Engine::Idle && state() != Engine::Empty )
+    {
+        const Engine::SimpleMetaBundle bundle = fetchMetaData();
+        if( bundle.title != m_currentBundle.title || bundle.artist != m_currentBundle.artist ) {
+            debug() << "Metadata received." << endl;
+            m_currentBundle = bundle;
+
+            XineEngine* p = const_cast<XineEngine*>( this );
+            p->emit metaData( bundle );
+        }
+    }
+
     return time;
 }
 
 uint
 XineEngine::length() const
 {
-    if( m_url.fileName().endsWith( ".ogg", false ) )
+    if ( !m_stream )
+       return 0;
+
+    // xine often delivers nonsense values for VBR files and such, so we only
+    // use the length for remote files
+
+    if( m_url.isLocalFile() )
         return 0;
 
-    int pos;
-    int time;
-    int length = 0;
+    else {
+        int pos;
+        int time;
+        int length = 0;
 
-    xine_get_pos_length( m_stream, &pos, &time, &length );
+        xine_get_pos_length( m_stream, &pos, &time, &length );
 
-    return length;
+        return length;
+    }
 }
 
 void
 XineEngine::seek( uint ms )
 {
+    if( !ensureStream() )
+        return;
+
     if( xine_get_param( m_stream, XINE_PARAM_SPEED ) == XINE_SPEED_PAUSE ) {
         // FIXME this is a xine API issue really, they need to add a seek function
         xine_play( m_stream, 0, (int)ms );
@@ -363,6 +475,8 @@ XineEngine::seek( uint ms )
 void
 XineEngine::setVolumeSW( uint vol )
 {
+    if ( !m_stream )
+       return;
     if( !s_fader )
         xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, static_cast<uint>( vol * m_preamp ) );
 }
@@ -370,6 +484,9 @@ XineEngine::setVolumeSW( uint vol )
 void
 XineEngine::setEqualizerEnabled( bool enable )
 {
+    if ( !m_stream )
+       return;
+
     m_equalizerEnabled = enable;
 
     if( !enable ) {
@@ -423,7 +540,7 @@ XineEngine::canDecode( const KURL &url ) const
     {
         char* exts = xine_get_file_extensions( m_xine );
         list = QStringList::split( ' ', exts );
-        delete [] exts; exts = NULL;
+        free( exts ); exts = NULL;
         //images
         list.remove("png");
         list.remove("jpg");
@@ -443,8 +560,19 @@ XineEngine::canDecode( const KURL &url ) const
         if(!list.contains("m4a"))
             list << "m4a";
     }
-    const QString path = url.path();
-    const QString ext  = path.mid( path.findRev( '.' ) + 1 ).lower();
+
+    if (url.protocol() == "cdda")
+        // play audio CDs pls
+        return true;
+
+    QString path = url.path();
+
+    // partial downloads from Konqi and other browsers
+    // tend to have a .part extension
+    if (path.endsWith( ".part" ))
+        path = path.left( path.length() - 5 );
+
+    const QString ext = path.mid( path.findRev( '.' ) + 1 ).lower();
 
     return list.contains( ext );
 }
@@ -452,12 +580,16 @@ XineEngine::canDecode( const KURL &url ) const
 const Engine::Scope&
 XineEngine::scope()
 {
-    if( !m_post || xine_get_status( m_stream ) != XINE_STATUS_PLAY )
+    if( !m_post || !m_stream || xine_get_status( m_stream ) != XINE_STATUS_PLAY )
        return m_scope;
 
     MyNode* const myList         = scope_plugin_list( m_post );
     metronom_t* const myMetronom = scope_plugin_metronom( m_post );
     const int myChannels         = scope_plugin_channels( m_post );
+    int scopeidx = 0;
+
+    if (myChannels > 2)
+       return m_scope;
 
     //prune the buffer list and update m_currentVpts
     timerEvent( 0 );
@@ -496,10 +628,12 @@ XineEngine::scope()
 
         for( int a, c; frame < n; ++frame, data16 += myChannels ) {
             for( a = c = 0; c < myChannels; ++c )
-                a += data16[c];
-
-            a /= myChannels;
-            m_scope[frame] = a;
+            {
+               // we now give interleaved pcm to the scope
+               m_scope[scopeidx++] = data16[c];
+               if (myChannels == 1) // duplicate mono samples
+                  m_scope[scopeidx++] = data16[c];
+            }
         }
 
         m_currentVpts = best_node->vpts_end;
@@ -514,10 +648,15 @@ XineEngine::scope()
 void
 XineEngine::timerEvent( QTimerEvent* )
 {
+   if ( !m_stream )
+      return;
+
    //here we prune the buffer list regularly
 
    MyNode *myList = scope_plugin_list( m_post );
 
+   if ( ! myList ) return;
+
    //we operate on a subset of the list for thread-safety
    MyNode * const first_node = myList->next;
    MyNode const * const list_end = myList;
@@ -560,7 +699,7 @@ XineEngine::customEvent( QCustomEvent *e )
 
     switch( e->type() )
     {
-    case 3000:
+    case 3000: //XINE_EVENT_UI_PLAYBACK_FINISHED
         emit trackEnded();
         break;
 
@@ -575,17 +714,9 @@ XineEngine::customEvent( QCustomEvent *e )
         break;
 
     case 3003: { //meta info has changed
-
         debug() << "Metadata received." << endl;
-
-        Engine::SimpleMetaBundle bundle;
-        bundle.title   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_TITLE ) );
-        bundle.artist  = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ARTIST ) );
-        bundle.album   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ALBUM ) );
-        bundle.comment = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_COMMENT ) );
-        bundle.genre   = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_GENRE ) );
-        bundle.bitrate = QString::number( xine_get_stream_info( m_stream, XINE_STREAM_INFO_BITRATE ) / 1000 );
-
+        const Engine::SimpleMetaBundle bundle = fetchMetaData();
+        m_currentBundle = bundle;
         emit metaData( bundle );
     }   break;
 
@@ -652,7 +783,15 @@ XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
         break;
 
     case XINE_EVENT_UI_PLAYBACK_FINISHED:
+        debug() << "XINE_EVENT_UI_PLAYBACK_FINISHED\n";
 
+        #ifdef XINE_PARAM_GAPLESS_SWITCH
+            if ( xine_check_version(1,1,1) && xe->m_url.isLocalFile() //Remote media break with gapless
+            //don't prepare for a track that isn't coming
+            && Playlist::instance()->isTrackAfter()
+            && !AmarokConfig::crossfade() )
+                xine_set_param( xe->m_stream, XINE_PARAM_GAPLESS_SWITCH, 1);
+        #endif
         //emit signal from GUI thread
         QApplication::postEvent( xe, new QCustomEvent(3000) );
         break;
@@ -791,11 +930,29 @@ XineEngine::XineEventListener( void *p, const xine_event_t* xineEvent )
     #undef xe
 }
 
+Engine::SimpleMetaBundle
+XineEngine::fetchMetaData() const
+{
+    Engine::SimpleMetaBundle bundle;
+    bundle.title      = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_TITLE ) );
+    bundle.artist     = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ARTIST ) );
+    bundle.album      = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_ALBUM ) );
+    bundle.comment    = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_COMMENT ) );
+    bundle.genre      = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_GENRE ) );
+    bundle.bitrate    = QString::number( xine_get_stream_info( m_stream, XINE_STREAM_INFO_AUDIO_BITRATE ) / 1000 );
+    bundle.samplerate = QString::number( xine_get_stream_info( m_stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE ) );
+    bundle.year       = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_YEAR ) );
+    bundle.tracknr    = QString::fromUtf8( xine_get_meta_info( m_stream, XINE_META_INFO_TRACK_NUMBER ) );
+
+    return bundle;
+}
+
+
 //////////////////
 /// class Fader
 //////////////////
 
-Fader::Fader( XineEngine *engine )
+Fader::Fader( XineEngine *engine, uint fadeMs )
    : QObject( engine )
    , QThread()
    , m_engine( engine )
@@ -804,6 +961,7 @@ Fader::Fader( XineEngine *engine )
    , m_increase( 0 )
    , m_port( engine->m_audioPort )
    , m_post( engine->m_post )
+   , m_fadeLength( fadeMs )
 {
     if( engine->makeNewStream() )
     {
@@ -835,83 +993,46 @@ Fader::~Fader()
      s_fader = 0;
 }
 
-struct fade_s {
-    int sleep;
-    uint volume;
-    xine_stream_t *stream;
-
-    fade_s( uint s, uint v, xine_stream_t *st ) : sleep( s ), volume( v ), stream( st ) {}
-};
-
-#include <list>
 void
 Fader::run()
 {
-    using std::list;
-
-    list<fade_s> data;
-
-    int sleeps[100];
-    for( uint v = 0; v < 100; ++v )
-        //the usleep time for this volume
-        sleeps[v] = int(120000.0 * (-log10( v+1 ) + 2));
-
-    for( int v = 99; v >= 0; --v )
-        data.push_back( fade_s( sleeps[v], v, m_decrease ) );
+    // do a volume change in 100 steps (or every 10ms)
+    uint stepsCount = m_fadeLength < 1000 ? m_fadeLength / 10 : 100;
+    uint stepSizeUs = (int)( 1000.0 * (float)m_fadeLength / (float)stepsCount );
 
+    QTime t;
+    t.start();
+    float mix = 0.0;
+    while ( mix < 1.0 )
     {
-        /**
-         * Here we try to make a list consisting of many small sleeps
-         * inbetween each volume increase/decrease
-         */
+        // sleep a constant amount of time
+        QThread::usleep( stepSizeUs );
 
-        uint v = 0;
-        int tu = 0;
-        int td = sleeps[0];
-        for( list<fade_s>::iterator it = data.begin(), end = data.end(); it != end; ++it ) {
-            tu += (*it).sleep;
+        // get volume (amarok main * equalizer preamp)
+        float vol = Engine::Base::makeVolumeLogarithmic( m_engine->m_volume ) * m_engine->m_preamp;
 
-            while ( tu > td ) {
-//                 kdDebug() << tu << ", " << td << " for v=" << v << endl;
-
-                //this is the sleeptime for the structure we are about to insert
-                const int newsleep = tu - td;
-
-                //first we need to update the sleep for the previous structure
-                list<fade_s>::iterator jt = it; --jt;
-                (*jt).sleep -= newsleep;
-
-                //insert the new structure for the increasing stream
-                data.insert( it, fade_s( newsleep, v, m_increase ) );
-
-//                 kdDebug() << "new: " << newsleep << endl;
-
-                //decrease the contextual volume
-                if( ++v > 99 )
-                    goto done;
-
-                //update td
-                td += sleeps[v];
-            }
-
-//             kdDebug() << tu << ", " << td << " for v=" << v << endl;
+        // compute the mix factor as the percentage of time spent since fade begun
+        float mix = (float)t.elapsed() / (float)m_fadeLength;
+        if ( mix > 1.0 )
+        {
+            if ( m_increase )
+                xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)vol );
+            break;
         }
 
-        done: ;
-    }
-
-    // perform the fading operations
-    list<fade_s>::iterator it, end;
-    for( it = data.begin(), end = data.end(); it != end && !m_engine->m_stopFader; ++it )
-    {
-//         debug() << "sleep: " << (*it).sleep << " volume: " << (*it).volume << endl;
-        if( (*it).sleep > 0 ) //FIXME
-           QThread::usleep( (*it).sleep );
-
-        float vol = Engine::Base::makeVolumeLogarithmic( m_engine->m_volume );
-        vol = vol * m_engine->m_preamp * ( (*it).volume * 0.01 );
-
-        xine_set_param( (*it).stream, XINE_PARAM_AUDIO_AMP_LEVEL, (uint) vol );
+        // change volume of streams (using dj-like cross-fade profile)
+        if ( m_decrease )
+        {
+            //xine_set_param( m_decrease, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)(vol * (1.0 - mix)) );  // linear
+            float v = 4.0 * (1.0 - mix) / 3.0;
+            xine_set_param( m_decrease, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)( v < 1.0 ? vol * v : vol ) );
+        }
+        if ( m_increase )
+        {
+            //xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)(vol * mix) );  //linear
+            float v = 4.0 * mix / 3.0;
+            xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)( v < 1.0 ? vol * v : vol ) );
+        }
     }
 
     //stop using cpu!
@@ -922,4 +1043,94 @@ Fader::run()
     deleteLater();
 }
 
+
+bool XineEngine::metaDataForUrl(const KURL &url, Engine::SimpleMetaBundle &b)
+{
+    bool result = false;
+    xine_stream_t* tmpstream = xine_stream_new(m_xine, NULL, NULL);
+    if (xine_open(tmpstream, QFile::encodeName(url.url()))) {
+        QString audioCodec = QString::fromUtf8(xine_get_meta_info(tmpstream, XINE_META_INFO_SYSTEMLAYER));
+
+        if (audioCodec == "CDDA") {
+            QString title = QString::fromUtf8(
+                xine_get_meta_info(tmpstream, XINE_META_INFO_TITLE));
+            if ((!title.isNull()) && (!title.isEmpty())) { //no meta info
+                b.title = title;
+                b.artist =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_ARTIST));
+                b.album =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_ALBUM));
+                b.genre =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_GENRE));
+                b.year =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_YEAR));
+                b.tracknr =
+                    QString::fromUtf8(
+                        xine_get_meta_info(tmpstream, XINE_META_INFO_TRACK_NUMBER));
+            } else {
+                b.title = QString(i18n("Track %1")).arg(url.filename());
+                b.album = i18n("AudioCD");
+            }
+        }
+
+        if (audioCodec == "CDDA" || audioCodec == "WAV") {
+            result = true;
+            int samplerate = xine_get_stream_info( tmpstream, XINE_STREAM_INFO_AUDIO_SAMPLERATE );
+
+            // xine would provide a XINE_STREAM_INFO_AUDIO_BITRATE, but unfortunately not for CDDA or WAV
+            // so we calculate the bitrate by our own
+            int bitsPerSample = xine_get_stream_info( tmpstream, XINE_STREAM_INFO_AUDIO_BITS );
+            int nbrChannels = xine_get_stream_info( tmpstream, XINE_STREAM_INFO_AUDIO_CHANNELS );
+            int bitrate = (samplerate * bitsPerSample * nbrChannels) / 1000;
+
+            b.bitrate = QString::number(bitrate);
+            b.samplerate = QString::number(samplerate);
+            int pos, time, length = 0;
+            xine_get_pos_length(tmpstream, &pos, &time, &length);
+            b.length = QString::number(length / 1000);
+        }
+        xine_close(tmpstream);
+    }
+    xine_dispose(tmpstream);
+    return result;
+}
+
+bool XineEngine::getAudioCDContents(const QString &device, KURL::List &urls)
+{
+    char **xine_urls = NULL;
+    int num;
+    int i = 0;
+
+    if (!device.isNull()) {
+        debug() << "xine-engine setting CD Device to: " << device << endl;
+        xine_cfg_entry_t config;
+        xine_config_lookup_entry(m_xine, "input.cdda_device", &config);
+        config.str_value = (char *)device.latin1();
+        xine_config_update_entry(m_xine, &config);
+    }
+
+    emit statusText(i18n("Getting AudioCD contents..."));
+
+    xine_urls = xine_get_autoplay_mrls(m_xine, "CD", &num);
+
+    if (xine_urls) {
+        while (xine_urls[i]) {
+            urls << KURL(xine_urls[i]);
+            ++i;
+        }
+    }
+    else emit statusText(i18n("Could not read AudioCD"));
+
+    return true;
+}
+
+bool XineEngine::flushBuffer()
+{
+    return false;
+}
+
 #include "xine-engine.moc"
