@@ -1,5 +1,7 @@
 // (c) 2004 Christian Muehlhaeuser <chris@chris.de>
 // (c) 2004 Sami Nieminen <sami.nieminen@iki.fi>
+// (c) 2006 Shane King <kde@dontletsstart.com>
+// (c) 2006 Iain Benson <iain@arctos.me.uk>
 // See COPYING file for licensing information.
 
 #define DEBUG_PREFIX "Scrobbler"
@@ -44,16 +46,17 @@ Scrobbler* Scrobbler::instance()
 Scrobbler::Scrobbler()
     : EngineObserver( EngineController::instance() )
     , m_similarArtistsJob( 0 )
-    , m_validForSending( true )
+    , m_validForSending( false )
+    , m_startPos( 0 )
     , m_submitter( new ScrobblerSubmitter() )
-    , m_item( 0 )
+    , m_item( new SubmitItem() )
 {}
 
 
 Scrobbler::~Scrobbler()
 {
-    delete m_submitter;
     delete m_item;
+    delete m_submitter;
 }
 
 
@@ -116,9 +119,9 @@ void Scrobbler::similarArtists( const QString & artist )
 
         QByteArray postData;
         QDataStream stream( postData, IO_WriteOnly );
-        stream.writeRawBytes( xmlRequest.utf8(), xmlRequest.length() );
+        stream.writeRawBytes( xmlRequest.utf8(), xmlRequest.utf8().length() );
 
-        m_similarArtistsBuffer = QString::null;
+        m_similarArtistsBuffer = QByteArray();
         m_artist = artist;
 
         m_similarArtistsJob = KIO::http_post( "http://ws.audioscrobbler.com/xmlrpc", postData, false );
@@ -200,10 +203,14 @@ void Scrobbler::audioScrobblerSimilarArtistsResult( KIO::Job* job ) //SLOT
 /**
  * Called when similar artists data is received for the TransferJob.
  */
-void Scrobbler::audioScrobblerSimilarArtistsData( KIO::Job*, const QByteArray& data ) //SLOT
+void Scrobbler::audioScrobblerSimilarArtistsData( KIO::Job* job, const QByteArray& data ) //SLOT
 {
-    // Append new chunk of string
-    m_similarArtistsBuffer += QString::fromUtf8( data, data.size() );
+    if ( m_similarArtistsJob != job )
+        return; //not the right job, so let's ignore it
+
+    uint oldSize = m_similarArtistsBuffer.size();
+    m_similarArtistsBuffer.resize( oldSize + data.size() );
+    memcpy( m_similarArtistsBuffer.data() + oldSize, data.data(), data.size() );
 }
 
 
@@ -212,44 +219,59 @@ void Scrobbler::audioScrobblerSimilarArtistsData( KIO::Job*, const QByteArray& d
  */
 void Scrobbler::engineNewMetaData( const MetaBundle& bundle, bool trackChanged )
 {
+    //debug() << "engineNewMetaData: " << bundle.artist() << ":" << bundle.album() << ":" << bundle.title() << ":" << trackChanged << endl;
     if ( !trackChanged )
     {
-        // Tags were changed, update them if not yet submitted.
-        // TODO: In this case submit could be enabled if the artist or title
-        // tag was missing initially and disabled submit
         debug() << "It's still the same track." << endl;
-        if ( m_item )
-        {
-            m_item->setArtist( bundle.artist() );
-            m_item->setAlbum( bundle.album() );
-            m_item->setTitle( bundle.title() );
-        }
+        m_item->setArtist( bundle.artist() );
+        m_item->setAlbum( bundle.album() );
+        m_item->setTitle( bundle.title() );
         return;
     }
 
+    //to work around xine bug, we have to explictly prevent submission the first few seconds of a track
+    //http://sourceforge.net/tracker/index.php?func=detail&aid=1401026&group_id=9655&atid=109655
+    m_timer.stop();
+    m_timer.start( 10000, true );
+
+    m_startPos = 0;
+
     // Plugins must not submit tracks played from online radio stations, even
     // if they appear to be providing correct metadata.
-    if ( bundle.streamUrl() != NULL ) {
+    if ( !bundle.streamUrl().isEmpty() )
+    {
         debug() << "Won't submit: It's a stream." << endl;
         m_validForSending = false;
     }
+    else if( bundle.podcastBundle() != NULL )
+    {
+        debug() << "Won't submit: It's a podcast." << endl;
+        m_validForSending = false;
+    }
     else
     {
-        delete m_item;
+        *m_item = SubmitItem( bundle.artist(), bundle.album(), bundle.title(), bundle.length() );
+        m_validForSending = true; // check length etc later
+    }
+}
 
-        // Songs with no artist or title data or a duration of less than
-        // 30 seconds must not be submitted.
-        if ( bundle.artist() != NULL && bundle.title() != NULL && bundle.length() >= 30 )
-        {
-            m_item = new SubmitItem( bundle.artist(), bundle.album(), bundle.title(), bundle.length() );
-            m_validForSending = true;
-        }
-        else
-        {
-            m_item = 0;
-            m_validForSending = false;
-            debug() << "Won't submit: No artist, no title, or less than 30 seconds." << endl;;
-        }
+
+/**
+ * Called when cue file detects track change
+ */
+void Scrobbler::subTrack( long currentPos, long startPos, long endPos )
+{
+    //debug() << "subTrack: " << currentPos << ":" << startPos << ":" << endPos << endl;
+    *m_item = SubmitItem( m_item->artist(), m_item->album(), m_item->title(), endPos - startPos );
+    if ( currentPos <= startPos + 2 ) // only submit if starting from the start of the track (need to allow 2 second difference for rounding/delay)
+    {
+        m_startPos = startPos * 1000;
+        m_validForSending = true;
+    }
+    else
+    {
+        debug() << "Won't submit: Detected cuefile jump to " << currentPos - startPos << " seconds into track." << endl;
+        m_validForSending = false;
     }
 }
 
@@ -259,7 +281,8 @@ void Scrobbler::engineNewMetaData( const MetaBundle& bundle, bool trackChanged )
  */
 void Scrobbler::engineTrackPositionChanged( long position, bool userSeek )
 {
-    if ( !m_validForSending || !m_item )
+    //debug() << "engineTrackPositionChanged: " << position << ":" << userSeek << endl;
+    if ( !m_validForSending )
         return;
 
     if ( userSeek )
@@ -269,12 +292,17 @@ void Scrobbler::engineTrackPositionChanged( long position, bool userSeek )
         return;
     }
 
+    if ( m_timer.isActive() )
+        return;
+
     // Each track must be submitted to the server when it is 50% or 240
     // seconds complete, whichever comes first.
-    if ( position > 240 * 1000 || position > 0.5 * m_item->length() * 1000 )
+    if ( position - m_startPos > 240 * 1000 || position - m_startPos > 0.5 * m_item->length() * 1000 )
     {
-        m_submitter->submitItem( m_item );
-        m_item = 0;
+        if ( m_item->valid() )
+            m_submitter->submitItem( new SubmitItem( *m_item ) );
+        else
+            debug() << "Won't submit: No artist, no title, or less than 30 seconds." << endl;
         m_validForSending = false;
     }
 }
@@ -285,22 +313,7 @@ void Scrobbler::engineTrackPositionChanged( long position, bool userSeek )
  */
 void Scrobbler::applySettings()
 {
-    bool handshakeNeeded = false;
-    if ( m_submitter->username() != AmarokConfig::scrobblerUsername() )
-    {
-        handshakeNeeded = true;
-    }
-    else if ( m_submitter->password() != AmarokConfig::scrobblerPassword() )
-    {
-        handshakeNeeded = true;
-    }
-
-    m_submitter->setEnabled( AmarokConfig::submitPlayedSongs() );
-    m_submitter->setUsername( AmarokConfig::scrobblerUsername() );
-    m_submitter->setPassword( AmarokConfig::scrobblerPassword() );
-
-    if ( handshakeNeeded )
-        m_submitter->handshake();
+    m_submitter->configure( AmarokConfig::scrobblerUsername(), AmarokConfig::scrobblerPassword(), AmarokConfig::submitPlayedSongs() );
 }
 
 
@@ -313,13 +326,14 @@ SubmitItem::SubmitItem(
     const QString& artist,
     const QString& album,
     const QString& title,
-    int length)
+    int length,
+    bool now)
 {
     m_artist = artist;
     m_album = album;
     m_title = title;
     m_length = length;
-    m_playStartTime = QDateTime::currentDateTime( Qt::UTC ).toTime_t();
+    m_playStartTime = now ? QDateTime::currentDateTime( Qt::UTC ).toTime_t() : 0;
 }
 
 
@@ -333,6 +347,13 @@ SubmitItem::SubmitItem( const QDomElement& element )
 }
 
 
+SubmitItem::SubmitItem()
+    : m_length( 0 )
+    , m_playStartTime( 0 )
+{
+}
+
+
 bool SubmitItem::operator==( const SubmitItem& item )
 {
     bool result = true;
@@ -389,8 +410,8 @@ QDomElement SubmitItem::toDomElement( QDomDocument& document ) const
 
 int SubmitQueue::compareItems( QPtrCollection::Item item1, QPtrCollection::Item item2 )
 {
-    SubmitItem *sItem1 = (SubmitItem*) item1;
-    SubmitItem *sItem2 = (SubmitItem*) item2;
+    SubmitItem *sItem1 = static_cast<SubmitItem*>( item1 );
+    SubmitItem *sItem2 = static_cast<SubmitItem*>( item2 );
     int result;
 
     if ( sItem1 == sItem2 )
@@ -416,7 +437,7 @@ int SubmitQueue::compareItems( QPtrCollection::Item item1, QPtrCollection::Item
 
 QString ScrobblerSubmitter::PROTOCOL_VERSION = "1.1";
 QString ScrobblerSubmitter::CLIENT_ID = "ark";
-QString ScrobblerSubmitter::CLIENT_VERSION = "0.1";
+QString ScrobblerSubmitter::CLIENT_VERSION = "1.4";
 QString ScrobblerSubmitter::HANDSHAKE_URL = "http://post.audioscrobbler.com/?hs=true";
 
 
@@ -426,34 +447,45 @@ ScrobblerSubmitter::ScrobblerSubmitter()
     , m_submitUrl( 0 )
     , m_challenge( 0 )
     , m_scrobblerEnabled( false )
+    , m_holdFakeQueue( false )
+    , m_inProgress( false )
+    , m_needHandshake( true )
     , m_prevSubmitTime( 0 )
     , m_interval( 0 )
+    , m_backoff( 0 )
+    , m_lastSubmissionFinishTime( 0 )
+    , m_fakeQueueLength( 0 )
 {
+    connect( &m_timer, SIGNAL(timeout()), this, SLOT(scheduledTimeReached()) );
     readSubmitQueue();
 }
 
 
 ScrobblerSubmitter::~ScrobblerSubmitter()
 {
+    // need to rescue current submit. This may meant it gets submitted twice,
+    // but last.fm handles that, and it's better than losing it when you quit
+    // while a submit is happening
+    for ( QPtrDictIterator<SubmitItem> it( m_ongoingSubmits ); it.current(); ++it )
+        m_submitQueue.inSort( it.current() );
+    m_ongoingSubmits.clear();
+
     saveSubmitQueue();
 
-    m_ongoingSubmits.setAutoDelete( true );
-    m_ongoingSubmits.clear();
     m_submitQueue.setAutoDelete( true );
     m_submitQueue.clear();
+    m_fakeQueue.setAutoDelete( true );
+    m_fakeQueue.clear();
 }
 
 
 /**
  * Performs handshake with Audioscrobbler.
  */
-void ScrobblerSubmitter::handshake()
+void ScrobblerSubmitter::performHandshake()
 {
-    if ( !canSubmit() )
-        return;
-
     QString handshakeUrl = QString::null;
-    uint currentTime = QDateTime::currentDateTime().toTime_t();
+    uint currentTime = QDateTime::currentDateTime( Qt::UTC ).toTime_t();
 
     if ( PROTOCOL_VERSION == "1.1" )
     {
@@ -464,8 +496,8 @@ void ScrobblerSubmitter::handshake()
         // &v=<clientver>
         // &u=<user>
         handshakeUrl =
-            HANDSHAKE_URL +
-            QString(
+                HANDSHAKE_URL +
+                QString(
                 "&p=%1"
                 "&c=%2"
                 "&v=%3"
@@ -487,8 +519,8 @@ void ScrobblerSubmitter::handshake()
         // &t=<unix_timestamp>
         // &a=<passcode>
         handshakeUrl =
-            HANDSHAKE_URL +
-            QString(
+                HANDSHAKE_URL +
+                QString(
                 "&p=%1"
                 "&c=%2"
                 "&v=%3"
@@ -501,7 +533,7 @@ void ScrobblerSubmitter::handshake()
                 .arg( m_username )
                 .arg( currentTime )
                 .arg( KMD5( KMD5( m_password.utf8() ).hexDigest() +
-                    currentTime ).hexDigest() );
+                currentTime ).hexDigest() );
     }
 
     else
@@ -512,9 +544,9 @@ void ScrobblerSubmitter::handshake()
 
     debug() << "Handshake url: " << handshakeUrl << endl;
 
-    m_prevSubmitTime = currentTime;
     m_submitResultBuffer = "";
 
+    m_inProgress = true;
     KIO::TransferJob* job = KIO::storedGet( handshakeUrl, false, false );
     connect( job, SIGNAL( result( KIO::Job* ) ), SLOT( audioScrobblerHandshakeResult( KIO::Job* ) ) );
 }
@@ -527,29 +559,24 @@ void ScrobblerSubmitter::handshake()
  */
 void ScrobblerSubmitter::submitItem( SubmitItem* item )
 {
-    if ( !canSubmit() )
-    {
-        if ( m_scrobblerEnabled )
-        {
-            // If scrobbling is enabled but can't submit for some reason,
-            // enqueue item.
-            enqueueItem( item );
-            announceSubmit( item, 1, false );
-        }
-        return;
-    }
-    else if ( m_challenge.isEmpty() )
-    {
-        enqueueItem( item );
-        announceSubmit( item, 1, false );
-        handshake();
-        return;
-    }
-    else
+    if ( m_scrobblerEnabled ) {
         enqueueItem( item );
 
+        if ( item->playStartTime() == 0 )
+            m_holdFakeQueue = true; // hold on to fake queue until we get it all and can compute when to submit
+        else if ( !schedule( false ) )
+            announceSubmit( item, 1, false ); // couldn't perform submit immediately, let user know
+    }
+}
+
+
+/**
+ * Flushes the submit queues
+ */
+void ScrobblerSubmitter::performSubmit()
+{
     QString data;
-    uint currentTime = QDateTime::currentDateTime().toTime_t();
+
     // Audioscrobbler accepts max 10 tracks on one submit.
     SubmitItem* items[10];
     for ( int submitCounter = 0; submitCounter < 10; submitCounter++ )
@@ -571,17 +598,28 @@ void ScrobblerSubmitter::submitItem( SubmitItem* item )
 
 
         data =
-            "u=" + KURL::encode_string_no_slash( m_username ) +
-            "&s=" +
+                "u=" + KURL::encode_string_no_slash( m_username ) +
+                "&s=" +
                 KURL::encode_string_no_slash( KMD5( KMD5( m_password.utf8() ).hexDigest() +
-                    m_challenge.utf8() ).hexDigest() );
+                m_challenge.utf8() ).hexDigest() );
 
         m_submitQueue.first();
         for ( int submitCounter = 0; submitCounter < 10; submitCounter++ )
         {
             SubmitItem* itemFromQueue = dequeueItem();
             if ( itemFromQueue == 0 )
-                break;
+            {
+                if( submitCounter == 0 )
+                {
+                    // this shouldn't happen, since we shouldn't be scheduled until we have something to do!
+                    debug() << "Nothing to submit!" << endl;
+                    return;
+                }
+                else
+                {
+                    break;
+                }
+            }
             else
                 data += "&";
 
@@ -592,12 +630,12 @@ void ScrobblerSubmitter::submitItem( SubmitItem* item )
             const QString count = QString::number( submitCounter );
 
             data +=
-                "a["  + count + "]=" + KURL::encode_string_no_slash( itemFromQueue->artist(), 106 /*utf-8*/ ) +
-                "&t[" + count + "]=" + KURL::encode_string_no_slash( itemFromQueue->title(), 106 /*utf-8*/ ) +
-                "&b[" + count + "]=" + KURL::encode_string_no_slash( itemFromQueue->album(), 106 /*utf-8*/ ) +
-                "&m[" + count + "]=" +
-                "&l[" + count + "]=" + QString::number( itemFromQueue->length() ) +
-                "&i[" + count + "]=" + KURL::encode_string_no_slash( playStartTime.toString( "yyyy-MM-dd hh:mm:ss" ) );
+                    "a["  + count + "]=" + KURL::encode_string_no_slash( itemFromQueue->artist(), 106 /*utf-8*/ ) +
+                    "&t[" + count + "]=" + KURL::encode_string_no_slash( itemFromQueue->title(), 106 /*utf-8*/ ) +
+                    "&b[" + count + "]=" + KURL::encode_string_no_slash( itemFromQueue->album(), 106 /*utf-8*/ ) +
+                    "&m[" + count + "]=" +
+                    "&l[" + count + "]=" + QString::number( itemFromQueue->length() ) +
+                    "&i[" + count + "]=" + KURL::encode_string_no_slash( playStartTime.toString( "yyyy-MM-dd hh:mm:ss" ) );
         }
     }
 
@@ -609,9 +647,9 @@ void ScrobblerSubmitter::submitItem( SubmitItem* item )
 
     debug() << "Submit data: " << data << endl;
 
-    m_prevSubmitTime = currentTime;
     m_submitResultBuffer = "";
 
+    m_inProgress = true;
     KIO::TransferJob* job = KIO::http_post( m_submitUrl, data.utf8(), false );
     job->addMetaData( "content-type", "Content-Type: application/x-www-form-urlencoded" );
 
@@ -633,31 +671,19 @@ void ScrobblerSubmitter::submitItem( SubmitItem* item )
 
 
 /**
- * Sets Audioscrobbler profile username.
+ * Configures the username/password and whether to scrobble
  */
-void ScrobblerSubmitter::setUsername( const QString& username )
+void ScrobblerSubmitter::configure( const QString& username, const QString& password, bool enabled )
 {
+    if ( username != m_username || password != m_password )
+        m_needHandshake = true;
+
     m_username = username;
-}
-
-
-/**
- * Sets Audioscrobbler profile password.
- */
-void ScrobblerSubmitter::setPassword( const QString& password )
-{
     m_password = password;
-}
-
-
-/**
- * Sets whether scrobbling is enabled.
- */
-void ScrobblerSubmitter::setEnabled( bool enabled )
-{
     m_scrobblerEnabled = enabled;
-
-    if ( !enabled )
+    if ( enabled )
+        schedule( false );
+    else
     {
         // If submit is disabled, clear submitqueue.
         m_ongoingSubmits.setAutoDelete( true );
@@ -666,17 +692,48 @@ void ScrobblerSubmitter::setEnabled( bool enabled )
         m_submitQueue.setAutoDelete( true );
         m_submitQueue.clear();
         m_submitQueue.setAutoDelete( false );
+        m_fakeQueue.setAutoDelete( true );
+        m_fakeQueue.clear();
+        m_fakeQueue.setAutoDelete( false );
+        m_fakeQueueLength = 0;
+        m_timer.stop();
     }
 }
 
 
+/**
+ * Sync from external device complete, can send them off
+ */
+void ScrobblerSubmitter::syncComplete()
+{
+    m_holdFakeQueue = false;
+    saveSubmitQueue();
+    schedule( false );
+}
+
+
+/**
+ * Called when timer set up in the schedule function goes off.
+ */
+void ScrobblerSubmitter::scheduledTimeReached()
+{
+    if ( m_needHandshake || m_challenge.isEmpty() )
+        performHandshake();
+    else
+        performSubmit();
+}
+
 /**
  * Called when handshake TransferJob has finished and data is received.
  */
 void ScrobblerSubmitter::audioScrobblerHandshakeResult( KIO::Job* job ) //SLOT
 {
+    m_prevSubmitTime = QDateTime::currentDateTime( Qt::UTC ).toTime_t();
+    m_inProgress = false;
+
     if ( job->error() ) {
         warning() << "KIO error! errno: " << job->error() << endl;
+        schedule( true );
         return;
     }
 
@@ -706,7 +763,7 @@ void ScrobblerSubmitter::audioScrobblerHandshakeResult( KIO::Job* job ) //SLOT
     // INTERVAL n (protocol 1.1)
     else if ( m_submitResultBuffer.startsWith( "UPDATE" ) )
     {
-        warning() << "A new version of amaroK is available" << endl;
+        warning() << "A new version of Amarok is available" << endl;
 
         m_challenge = m_submitResultBuffer.section( "\n", 1, 1 );
         m_submitUrl = m_submitResultBuffer.section( "\n", 2, 2 );
@@ -741,6 +798,8 @@ void ScrobblerSubmitter::audioScrobblerHandshakeResult( KIO::Job* job ) //SLOT
         warning() << "Unknown handshake response: " << m_submitResultBuffer << endl;
 
     debug() << "Handshake result parsed: challenge=" << m_challenge << ", submitUrl=" << m_submitUrl << endl;
+
+    schedule( m_challenge.isEmpty() ); // schedule to submit or re-attempt handshake
 }
 
 
@@ -749,6 +808,9 @@ void ScrobblerSubmitter::audioScrobblerHandshakeResult( KIO::Job* job ) //SLOT
  */
 void ScrobblerSubmitter::audioScrobblerSubmitResult( KIO::Job* job ) //SLOT
 {
+    m_prevSubmitTime = QDateTime::currentDateTime( Qt::UTC ).toTime_t();
+    m_inProgress = false;
+
     if ( job->error() ) {
         warning() << "KIO error! errno: " << job->error() << endl;
         enqueueJob( job );
@@ -796,8 +858,8 @@ void ScrobblerSubmitter::audioScrobblerSubmitResult( KIO::Job* job ) //SLOT
         if ( interval.startsWith( "INTERVAL" ) )
             m_interval = interval.mid( 9 ).toUInt();
 
+        m_challenge = QString::null;
         enqueueJob( job );
-        handshake();
     }
     else
     {
@@ -824,14 +886,9 @@ void ScrobblerSubmitter::audioScrobblerSubmitData(
 bool ScrobblerSubmitter::canSubmit() const
 {
     if ( !m_scrobblerEnabled || m_username.isEmpty() || m_password.isEmpty() )
-        return false;
-
-    if ( m_interval != 0 )
     {
-        uint currentTime = QDateTime::currentDateTime().toTime_t();
-        if ( ( currentTime - m_prevSubmitTime ) < m_interval )
-            // Not enough time passed since previous handshake/submit
-            return false;
+        debug() << "Unable to submit - no uname/pass or disabled" << endl;
+        return false;
     }
 
     return true;
@@ -845,6 +902,21 @@ void ScrobblerSubmitter::enqueueItem( SubmitItem* item )
 {
     // Maintain max size of the queue, Audioscrobbler won't accept too old
     // submissions anyway.
+    m_fakeQueue.first();
+    for ( uint size = m_fakeQueue.count() + m_submitQueue.count(); size >= 500; size-- )
+    {
+        SubmitItem* itemFromQueue = m_fakeQueue.getFirst();
+        m_fakeQueue.removeFirst();
+
+        if ( itemFromQueue )
+        {
+            debug() << "Dropping " << itemFromQueue->artist()
+                    << " - " << itemFromQueue->title() << " from fake queue" << endl;
+            m_fakeQueueLength -= itemFromQueue->length();
+        }
+
+        delete itemFromQueue;
+    }
     m_submitQueue.first();
     for ( uint size = m_submitQueue.count(); size >= 500; size-- )
     {
@@ -856,10 +928,21 @@ void ScrobblerSubmitter::enqueueItem( SubmitItem* item )
         delete itemFromQueue;
     }
 
-    m_submitQueue.inSort( item );
+    if( item->playStartTime() == 0 )
+    {
+        m_fakeQueue.inSort( item );
+        m_fakeQueueLength += item->length();
+    }
+    else
+    {
+        m_submitQueue.inSort( item );
+    }
 
-    // Save submit queue to disk so it is more uptodate in case of crash.
-    saveSubmitQueue();
+    if( !m_holdFakeQueue )
+    {
+       // Save submit queue to disk so it is more uptodate in case of crash.
+       saveSubmitQueue();
+    }
 }
 
 
@@ -868,7 +951,49 @@ void ScrobblerSubmitter::enqueueItem( SubmitItem* item )
  */
 SubmitItem* ScrobblerSubmitter::dequeueItem()
 {
-    SubmitItem* item = m_submitQueue.take();
+    SubmitItem* item = 0;
+    if( m_lastSubmissionFinishTime > 0 && !m_holdFakeQueue && m_fakeQueue.getFirst() )
+    {
+        uint limit = QDateTime::currentDateTime( Qt::UTC ).toTime_t();
+
+        if ( m_submitQueue.getFirst() )
+            if ( m_submitQueue.getFirst()->playStartTime() <= limit )
+                limit = m_submitQueue.getFirst()->playStartTime();
+
+        if( m_lastSubmissionFinishTime + m_fakeQueue.getFirst()->length() <= limit )
+        {
+            m_fakeQueue.first();
+            item = m_fakeQueue.take();
+            // don't backdate earlier than we have to
+            if( m_lastSubmissionFinishTime + m_fakeQueueLength < limit )
+                item->m_playStartTime = limit - m_fakeQueueLength;
+            else
+                item->m_playStartTime = m_lastSubmissionFinishTime;
+            m_fakeQueueLength -= item->length();
+        }
+    }
+
+    if( !item )
+    {
+        m_submitQueue.first();
+        item = m_submitQueue.take();
+    }
+
+    if( item )
+    {
+        if( item->playStartTime() < m_lastSubmissionFinishTime )
+        {
+//            debug() << "play times screwed up? - " << item->artist() << " - " << item->title() << ": " << item->playStartTime() << " < " << m_lastSubmissionFinishTime << endl;
+        }
+        int add = 30;
+        if( item->length() / 2 + 1 > add )
+            add = item->length() / 2 + 1;
+        if( item->playStartTime() + add > m_lastSubmissionFinishTime )
+            m_lastSubmissionFinishTime = item->playStartTime() + add;
+
+        // Save submit queue to disk so it is more uptodate in case of crash.
+        saveSubmitQueue();
+    }
 
     return item;
 }
@@ -891,7 +1016,10 @@ void ScrobblerSubmitter::enqueueJob( KIO::Job* job )
     }
     m_submitQueue.first();
 
-    announceSubmit( lastItem, counter, false );
+    if( lastItem )
+        announceSubmit( lastItem, counter, false );
+
+    schedule( true ); // arrange to flush queue after failure
 }
 
 
@@ -911,13 +1039,13 @@ void ScrobblerSubmitter::finishJob( KIO::Job* job )
             firstItem = item;
         else
             delete item;
-
-        m_submitQueue.remove( item );
     }
-    m_submitQueue.first();
 
-    announceSubmit( firstItem, counter, true );
+    if( firstItem )
+        announceSubmit( firstItem, counter, true );
     delete firstItem;
+
+    schedule( false ); // arrange to flush rest of queue
 }
 
 
@@ -928,8 +1056,7 @@ void ScrobblerSubmitter::finishJob( KIO::Job* job )
  * @param tracks Amount of tracks that were submitted
  * @param success Indicates if the submission was successful or not
  */
-void ScrobblerSubmitter::announceSubmit(
-    SubmitItem *item, int tracks, bool success ) const
+void ScrobblerSubmitter::announceSubmit( SubmitItem *item, int tracks, bool success ) const
 {
     QString _long, _short;
 
@@ -961,10 +1088,11 @@ void ScrobblerSubmitter::announceSubmit(
         }
     }
 
-    if ( m_submitQueue.count() > 0 )
+    if ( m_submitQueue.count() + m_fakeQueue.count() > 0 )
     {
         _long += "<p>";
-        _long += i18n( "One track still in queue", "%n tracks still in queue", m_submitQueue.count() );
+        _long += i18n( "One track still in queue", "%n tracks still in queue",
+                m_submitQueue.count() + m_fakeQueue.count() );
     }
 
     amaroK::StatusBar::instance()->shortLongMessage( _short, _long );
@@ -981,10 +1109,14 @@ void ScrobblerSubmitter::saveSubmitQueue()
         return;
     }
 
+    if ( m_lastSubmissionFinishTime == 0 )
+        m_lastSubmissionFinishTime = QDateTime::currentDateTime( Qt::UTC ).toTime_t();
+
     QDomDocument newdoc;
     QDomElement submitQueue = newdoc.createElement( "submit" );
-    submitQueue.setAttribute( "product", "amaroK" );
+    submitQueue.setAttribute( "product", "Amarok" );
     submitQueue.setAttribute( "version", APP_VERSION );
+    submitQueue.setAttribute( "lastSubmissionFinishTime", m_lastSubmissionFinishTime );
 
     m_submitQueue.first();
     for ( uint idx = 0; idx < m_submitQueue.count(); idx++ )
@@ -993,6 +1125,13 @@ void ScrobblerSubmitter::saveSubmitQueue()
         QDomElement i = item->toDomElement( newdoc );
         submitQueue.appendChild( i );
     }
+    m_fakeQueue.first();
+    for ( uint idx = 0; idx < m_fakeQueue.count(); idx++ )
+    {
+        SubmitItem *item = m_fakeQueue.at( idx );
+        QDomElement i = item->toDomElement( newdoc );
+        submitQueue.appendChild( i );
+    }
 
     QDomNode submitNode = newdoc.importNode( submitQueue, true );
     newdoc.appendChild( submitNode );
@@ -1026,6 +1165,12 @@ void ScrobblerSubmitter::readSubmitQueue()
         return;
     }
 
+    uint last = 0;
+    if( d.namedItem( "submit" ).isElement() )
+        last = d.namedItem( "submit" ).toElement().attribute( "lastSubmissionFinishTime" ).toUInt();
+    if(last && last > m_lastSubmissionFinishTime)
+        m_lastSubmissionFinishTime = last;
+
     const QString ITEM( "item" ); //so we don't construct these QStrings all the time
 
     for( QDomNode n = d.namedItem( "submit" ).firstChild(); !n.isNull() && n.nodeName() == ITEM; n = n.nextSibling() )
@@ -1035,4 +1180,69 @@ void ScrobblerSubmitter::readSubmitQueue()
 }
 
 
+/**
+ * Schedules an Audioscrobbler handshake or submit as required.
+ * Returns true if an immediate submit was possible
+ */
+bool ScrobblerSubmitter::schedule( bool failure )
+{
+    m_timer.stop();
+    if ( m_inProgress || !canSubmit() )
+        return false;
+
+    uint when, currentTime = QDateTime::currentDateTime( Qt::UTC ).toTime_t();
+    if ( currentTime - m_prevSubmitTime > m_interval )
+        when = 0;
+    else
+        when = m_interval - ( currentTime - m_prevSubmitTime );
+
+    if ( failure )
+    {
+        m_backoff = kMin( kMax( m_backoff * 2, unsigned( MIN_BACKOFF ) ), unsigned( MAX_BACKOFF ) );
+        when = kMax( m_backoff, m_interval );
+    }
+    else
+        m_backoff = 0;
+
+    if ( m_needHandshake || m_challenge.isEmpty() )
+    {
+        m_challenge = QString::null;
+        m_needHandshake = false;
+
+        if ( when == 0 )
+        {
+            debug() << "Performing immediate handshake" << endl;
+            performHandshake();
+        }
+        else
+        {
+            debug() << "Performing handshake in " << when << " seconds" << endl;
+            m_timer.start( when * 1000, true );
+        }
+    }
+    else if ( !m_submitQueue.isEmpty() || !m_holdFakeQueue && !m_fakeQueue.isEmpty() )
+    {
+        // if we only have stuff in the fake queue, we need to only schedule for when we can actually do something with it
+        if ( m_submitQueue.isEmpty() && m_lastSubmissionFinishTime + m_fakeQueue.getFirst()->length() > currentTime + when )
+            when = m_lastSubmissionFinishTime + m_fakeQueue.getFirst()->length() - currentTime;
+
+        if ( when == 0 )
+        {
+            debug() << "Performing immediate submit" << endl;
+            performSubmit();
+            return true;
+        }
+        else
+        {
+            debug() << "Performing submit in " << when << " seconds" << endl;
+            m_timer.start( when * 1000, true );
+        }
+    } else {
+        debug() << "Nothing to schedule" << endl;
+    }
+
+    return false;
+}
+
+
 #include "scrobbler.moc"
