@@ -1,8 +1,13 @@
-// (c) Pierpaolo Di Panfilo 2004
+/// (c) Pierpaolo Di Panfilo 2004
 // (c) Alexandre Pereira de Oliveira 2005
+// (c) 2005 Isaiah Damron <xepo@trifault.net>
+// (c) 2006 Peter C. Ndikuwera <pndiku@gmail.com>
 // See COPYING file for licensing information
 
+#define DEBUG_PREFIX "SmartPlaylistEditor"
+
 #include "amarok.h" //foreach
+#include "debug.h"
 #include "collectiondb.h"
 #include "metabundle.h"
 #include "smartplaylisteditor.h"
@@ -25,6 +30,29 @@
 #include <qvbox.h>
 #include <qvgroupbox.h>
 
+enum Fields
+{
+    FArtist = 0,
+    FComposer,
+    FAlbum,
+    FGenre,
+    FTitle,
+    FLength,
+    FTrack,
+    FYear,
+    FComment,
+    FPlayCounter,
+    FScore,
+    FRating,
+    FFirstPlay,
+    FLastPlay,
+    FModfiedDate,
+    FFilePath,
+    FBPM,
+    FMountPoint,
+    FBitRate
+};
+
 
 QStringList m_fields;
 QStringList m_dbFields;
@@ -34,38 +62,57 @@ QStringList m_expandableDbFields;
 
 
 SmartPlaylistEditor::SmartPlaylistEditor( QString defaultName, QWidget *parent, const char *name )
-    : KDialogBase( parent, name, true, i18n("Create Smart Playlist"), Ok|Cancel)
+    : KDialogBase( parent, name, true, i18n("Create Smart Playlist"),
+      Ok|Cancel, Ok, true )
 {
     init(defaultName);
-    addCriteria();
+    addCriteriaAny();
+    addCriteriaAll();
 }
 
 
 SmartPlaylistEditor::SmartPlaylistEditor( QWidget *parent, QDomElement xml, const char *name)
-    : KDialogBase( parent, name, true, i18n("Edit Smart Playlist"), Ok|Cancel)
+    : KDialogBase( parent, name, true, i18n("Edit Smart Playlist"),
+      Ok|Cancel, Ok, true )
 {
     init( xml.attribute( "name" ) );
     // matches
     QDomNodeList matchesList =  xml.elementsByTagName( "matches" );
-    if ( matchesList.count() ) {
-        m_matchCheck->setChecked( true );
+    bool matchedANY = false, matchedALL = false;
 
-        QDomElement matches = matchesList.item(0).toElement(); // we only allow one matches node
+    m_matchAllCheck->setChecked( true );
+    m_matchAnyCheck->setChecked( true );
+
+    for (int i = 0, m = matchesList.count(); i<m; i++) {
+        QDomElement matches = matchesList.item(i).toElement();
         QDomNodeList criteriaList =  matches.elementsByTagName( "criteria" );
-        for (int i = 0, c=criteriaList.count() ; i<c; ++i ) {
 
-            QDomElement criteria = criteriaList.item(i).toElement();
-            addCriteria( criteria );
+        if ( criteriaList.count() ) {
+            for (int j = 0, c=criteriaList.count() ; j<c; ++j ) {
+                QDomElement criteria = criteriaList.item(j).toElement();
+
+                if (matches.attribute( "glue" ) == "OR") {
+                    matchedANY = true;
+                    addCriteriaAny( criteria );
+                }
+                else {
+                    matchedALL = true;
+                    addCriteriaAll( criteria );
+                }
+            }
         }
-        if ( matches.attribute( "glue" ) == "AND" )
-            m_matchCombo->setCurrentItem( 0 );
-        else
-            m_matchCombo->setCurrentItem( 1 );
     }
-    else {
-        addCriteria();
-        m_matchCheck->setChecked( false );
+
+    if ( !matchedALL ) {
+        addCriteriaAll();
+        m_matchAllCheck->setChecked( false );
     }
+
+    if ( !matchedANY ) {
+        m_matchAnyCheck->setChecked( false );
+        addCriteriaAny( );
+    }
+
     // orderby
     QDomNodeList orderbyList =  xml.elementsByTagName( "orderby" );
     if ( orderbyList.count() ) {
@@ -74,6 +121,7 @@ SmartPlaylistEditor::SmartPlaylistEditor( QWidget *parent, QDomElement xml, cons
 
         //random is always the last one.
         int dbfield = orderby.attribute( "field" ) == "random" ? m_dbFields.count() : m_dbFields.findIndex( orderby.attribute( "field" ) );
+
         m_orderCombo->setCurrentItem( dbfield );
         updateOrderTypes( dbfield );
         if ( orderby.attribute( "order" ) == "DESC" || orderby.attribute( "order" ) == "weighted" )
@@ -104,21 +152,24 @@ void SmartPlaylistEditor::init(QString defaultName)
     makeVBoxMainWidget();
 
     m_fields.clear();
-    m_fields << i18n("Artist") << i18n("Album") << i18n("Genre") << i18n("Title") << i18n("Length") << i18n("Track #") << i18n("Year")
-             << i18n("Comment") << i18n("Play Counter") << i18n("Score") << i18n("First Play") << i18n("Last Play")
-             << i18n("Modified Date") << i18n("File Path");
+    m_fields << i18n("Artist") << i18n("Composer") << i18n("Album") << i18n("Genre") << i18n("Title") << i18n("Length")
+             << i18n("Track #") << i18n("Year") << i18n("Comment") << i18n("Play Counter")
+             << i18n("Score") << i18n( "Rating" ) << i18n("First Play")
+             << i18n("Last Play") << i18n("Modified Date") << i18n("File Path")
+             << i18n("BPM") << i18n("Mount Point") << i18n( "Bitrate" );
 
     m_dbFields.clear();
-    m_dbFields << "artist.name" << "album.name" << "genre.name" << "tags.title" << "tags.length"
+    m_dbFields << "artist.name" << "composer.name" << "album.name" << "genre.name" << "tags.title" << "tags.length"
                << "tags.track" << "year.name" << "tags.comment" << "statistics.playcounter"
-               << "statistics.percentage" << "statistics.createdate" << "statistics.accessdate"
-               << "tags.createdate" << "tags.url";
+               << "statistics.percentage" << "statistics.rating" << "statistics.createdate"
+               << "statistics.accessdate" << "tags.createdate" << "tags.url"
+               << "tags.bpm" << "devices.lastmountpoint" << "tags.bitrate";
 
     m_expandableFields.clear();
-    m_expandableFields << i18n("Artist") << i18n("Album") << i18n("Genre") <<  i18n("Year");
+    m_expandableFields << i18n("Artist") << i18n("Composer") << i18n("Album") << i18n("Genre") <<  i18n("Year");
 
     m_expandableDbFields.clear();
-    m_expandableDbFields << "artist.name" << "album.name" << "genre.name" << "year.name";
+    m_expandableDbFields << "artist.name" << "composer.name" << "album.name" << "genre.name" << "year.name";
 
     QHBox *hbox = new QHBox( mainWidget() );
     hbox->setSpacing( 5 );
@@ -128,18 +179,21 @@ void SmartPlaylistEditor::init(QString defaultName)
     QFrame *sep = new QFrame( mainWidget() );
     sep->setFrameStyle( QFrame::HLine | QFrame::Sunken );
 
-    //match box
-    QHBox *matchBox = new QHBox( mainWidget() );
-    m_matchCheck = new QCheckBox( i18n("Match the following condition" ), matchBox );
-    m_matchCheck->setChecked( true );
-    m_matchCombo = new KComboBox( matchBox );
-    m_matchCombo->insertItem( i18n("All") );
-    m_matchCombo->insertItem( i18n("Any") );
-    m_matchLabel = new QLabel( i18n(" of the following conditions"), matchBox );
-    matchBox->setStretchFactor( new QWidget( matchBox ), 1 );
+    //match box (any)
+    QHBox *matchAnyBox = new QHBox( mainWidget() );
+    m_matchAnyCheck = new QCheckBox( i18n("Match Any of the following conditions" ), matchAnyBox );
+    matchAnyBox->setStretchFactor( new QWidget( matchAnyBox ), 1 );
 
     //criteria box
-    m_criteriaGroupBox = new QVGroupBox( QString::null, mainWidget() );
+    m_criteriaAnyGroupBox = new QVGroupBox( QString::null, mainWidget() );
+
+    //match box (all)
+    QHBox *matchAllBox = new QHBox( mainWidget() );
+    m_matchAllCheck = new QCheckBox( i18n("Match All of the following conditions" ), matchAllBox );
+    matchAllBox->setStretchFactor( new QWidget( matchAllBox ), 1 );
+
+    //criteria box
+    m_criteriaAllGroupBox = new QVGroupBox( QString::null, mainWidget() );
 
     //order box
     QHBox *hbox2 = new QHBox( mainWidget() );
@@ -179,12 +233,16 @@ void SmartPlaylistEditor::init(QString defaultName)
     //add stretch
     static_cast<QHBox *>(mainWidget())->setStretchFactor(new QWidget(mainWidget()), 1);
 
-    connect( m_matchCheck, SIGNAL( toggled(bool) ), m_criteriaGroupBox, SLOT( setEnabled(bool) ) );
+    connect( m_matchAnyCheck, SIGNAL( toggled(bool) ), m_criteriaAnyGroupBox, SLOT( setEnabled(bool) ) );
+    connect( m_matchAllCheck, SIGNAL( toggled(bool) ), m_criteriaAllGroupBox, SLOT( setEnabled(bool) ) );
     connect( m_orderCheck, SIGNAL( toggled(bool) ), orderBox, SLOT( setEnabled(bool) ) );
     connect( m_limitCheck, SIGNAL( toggled(bool) ), limitBox, SLOT(  setEnabled(bool) ) );
     connect( m_expandCheck, SIGNAL( toggled(bool) ), expandBox, SLOT( setEnabled(bool) ) );
     connect( m_orderCombo, SIGNAL( activated(int) ), this, SLOT( updateOrderTypes(int) ) );
 
+    m_criteriaAnyGroupBox->setEnabled( false );
+    m_criteriaAllGroupBox->setEnabled( false );
+
     orderBox->setEnabled( false );
     limitBox->setEnabled( false );
     expandBox->setEnabled( false );
@@ -195,32 +253,52 @@ void SmartPlaylistEditor::init(QString defaultName)
 }
 
 
-void SmartPlaylistEditor::addCriteria()
+void SmartPlaylistEditor::addCriteriaAny()
 {
-    CriteriaEditor *criteria = new CriteriaEditor( this, m_criteriaGroupBox );
-    m_criteriaEditorList.append( criteria );
-    updateMatchWidgets();
-    m_criteriaEditorList.first()->enableRemove( m_criteriaEditorList.count() > 1 );
+    CriteriaEditor *criteria= new CriteriaEditor( this, m_criteriaAnyGroupBox, criteriaAny );
+    m_criteriaEditorAnyList.append( criteria );
+    m_criteriaEditorAnyList.first()->enableRemove( m_criteriaEditorAnyList.count() > 1 );
 }
 
-void SmartPlaylistEditor::addCriteria( QDomElement &xml )
+void SmartPlaylistEditor::addCriteriaAll()
 {
-    CriteriaEditor *criteria = new CriteriaEditor( this, m_criteriaGroupBox, xml );
-    m_criteriaEditorList.append( criteria );
-    updateMatchWidgets();
-    m_criteriaEditorList.first()->enableRemove( m_criteriaEditorList.count() > 1 );
+    CriteriaEditor *criteria= new CriteriaEditor( this, m_criteriaAllGroupBox, criteriaAll );
+    m_criteriaEditorAllList.append( criteria );
+    m_criteriaEditorAllList.first()->enableRemove( m_criteriaEditorAllList.count() > 1 );
 }
 
-
-void SmartPlaylistEditor::removeCriteria( CriteriaEditor *criteria )
+void SmartPlaylistEditor::addCriteriaAny( QDomElement &xml )
 {
-    m_criteriaEditorList.remove( criteria );
+    CriteriaEditor *criteria = new CriteriaEditor( this, m_criteriaAnyGroupBox, criteriaAny, xml );
+    m_criteriaEditorAnyList.append( criteria );
+    m_criteriaEditorAnyList.first()->enableRemove( m_criteriaEditorAnyList.count() > 1 );
+}
+
+void SmartPlaylistEditor::addCriteriaAll( QDomElement &xml )
+{
+    CriteriaEditor *criteria = new CriteriaEditor( this, m_criteriaAllGroupBox, criteriaAll, xml );
+    m_criteriaEditorAllList.append( criteria );
+    m_criteriaEditorAllList.first()->enableRemove( m_criteriaEditorAllList.count() > 1 );
+}
+
+void SmartPlaylistEditor::removeCriteriaAny( CriteriaEditor *criteria )
+{
+    m_criteriaEditorAnyList.remove( criteria );
     criteria->deleteLater();
-    updateMatchWidgets();
     resize( size().width(), sizeHint().height() );
 
-    if( m_criteriaEditorList.count() == 1 )
-        m_criteriaEditorList.first()->enableRemove( false );
+    if( m_criteriaEditorAnyList.count() == 1 )
+	m_criteriaEditorAnyList.first()->enableRemove( false );
+}
+
+void SmartPlaylistEditor::removeCriteriaAll( CriteriaEditor *criteria )
+{
+    m_criteriaEditorAllList.remove( criteria );
+    criteria->deleteLater();
+    resize( size().width(), sizeHint().height() );
+
+    if( m_criteriaEditorAllList.count() == 1 )
+	m_criteriaEditorAllList.first()->enableRemove( false );
 }
 
 void SmartPlaylistEditor::updateOrderTypes( int index )
@@ -258,19 +336,30 @@ QDomElement SmartPlaylistEditor::result() {
 
     nodeE.appendChild( smartplaylist );
     // Matches
-    if( m_matchCheck->isChecked() ) {
+    if( m_matchAnyCheck->isChecked() ) {
         QDomElement matches = doc.createElement("matches");
         smartplaylist.appendChild( matches );
         // Iterate through all criteria list
-        CriteriaEditor *criteriaeditor = m_criteriaEditorList.first();
-        for( int i=0; criteriaeditor; criteriaeditor = m_criteriaEditorList.next(), ++i ) {
+        CriteriaEditor *criteriaeditor = m_criteriaEditorAnyList.first();
+        for( int i=0; criteriaeditor; criteriaeditor = m_criteriaEditorAnyList.next(), ++i ) {
             matches.appendChild( doc.importNode( criteriaeditor->getDomSearchCriteria( doc ), true ) );
         }
-        if ( m_criteriaEditorList.count() > 1 ) {
-            matches.setAttribute( "glue", m_matchCombo->currentItem() == 0 ? "AND" : "OR" );
-        }
+        matches.setAttribute( "glue",  "OR" );
         smartplaylist.appendChild( matches );
     }
+
+    if( m_matchAllCheck->isChecked() ) {
+        QDomElement matches = doc.createElement("matches");
+        smartplaylist.appendChild( matches );
+        // Iterate through all criteria list
+        CriteriaEditor *criteriaeditor = m_criteriaEditorAllList.first();
+        for( int i=0; criteriaeditor; criteriaeditor = m_criteriaEditorAllList.next(), ++i ) {
+            matches.appendChild( doc.importNode( criteriaeditor->getDomSearchCriteria( doc ), true ) );
+        }
+        matches.setAttribute( "glue",  "AND" );
+        smartplaylist.appendChild( matches );
+    }
+
     // Order By
     if( m_orderCheck->isChecked() ) {
         QDomElement orderby = doc.createElement("orderby");
@@ -302,50 +391,78 @@ QDomElement SmartPlaylistEditor::result() {
 
 void SmartPlaylistEditor::buildQuery()
 {
+    DEBUG_BLOCK
+    //FIXME max: make sure sql queries are correct
 
-    QString joins = "tags INNER JOIN year ON year.id=tags.year INNER JOIN genre ON genre.id=tags.genre"
-                    " INNER JOIN artist ON artist.id=tags.artist INNER JOIN album ON album.id=tags.album";
+    QString joins = "tags LEFT JOIN year ON year.id=tags.year LEFT JOIN genre ON genre.id=tags.genre"
+                    " LEFT JOIN artist ON artist.id=tags.artist LEFT JOIN album ON album.id=tags.album"
+                    " LEFT JOIN composer ON composer.id=tags.composer";
     QString whereStr;
     QString criteriaListStr;
     QString orderStr;
     QString limitStr;
 
     //where expression
-    if( m_matchCheck->isChecked() ) {
-        criteriaListStr += " (";
+    if( m_matchAnyCheck->isChecked() || m_matchAllCheck->isChecked() ) {
+        int i = 0;
 
-        CriteriaEditor *criteria = m_criteriaEditorList.first();
-        for( int i=0; criteria; criteria = m_criteriaEditorList.next(), i++ ) {
+        if( m_matchAnyCheck->isChecked() ) {
+            criteriaListStr += "( (";
 
-            QString str = criteria->getSearchCriteria();
-            //add the table used in the search expression to tables
-            QString table = str.left( str.find('.') );
-             if( !joins.contains( table ) ) {
-                // that makes it possible to search for tracks never played. it looks ugly but is works
-                if( str.contains(" OR statistics.playcounter IS NULL") || str.contains(" OR statistics.percentage IS NULL") )
-                    joins += " LEFT JOIN statistics ON statistics.url=tags.url";
-                else
-                    joins += " INNER JOIN statistics ON statistics.url=tags.url";
+            CriteriaEditor *criteria = m_criteriaEditorAnyList.first();
+            for( i=0; criteria; criteria = m_criteriaEditorAnyList.next(), i++ ) {
+
+                QString str = criteria->getSearchCriteria();
+                if( str.contains( "statistics." ) && !joins.contains( "statistics" ) )
+                    joins += " LEFT JOIN statistics ON statistics.url=tags.url AND statistics.deviceid=tags.deviceid";
+                if( str.contains( "devices." ) && !joins.contains( "devices" ) )
+                    joins += " LEFT JOIN devices ON tags.deviceid=devices.id";
+
+                if( i ) //multiple conditions
+                    str.prepend( " OR (");
+
+                criteriaListStr += str+")";
             }
-            if( i ) { //multiple conditions
-                QString op = m_matchCombo->currentItem() == 0 ? "AND" : "OR";
-                str.prepend( " " + op + " (");
 
-            }
-            criteriaListStr += str+")";
+            criteriaListStr += " )"; // we want our ORs all in one bracket. :-)
         }
-        whereStr = " WHERE" + criteriaListStr;
+
+        if( m_matchAllCheck->isChecked() ) {
+            if ( i ) // conditions exist from above
+                criteriaListStr += " AND ";
+
+            criteriaListStr += "( (";
+
+            CriteriaEditor *criteria2 = m_criteriaEditorAllList.first();
+            for( i=0; criteria2; criteria2 = m_criteriaEditorAllList.next(), i++ ) {
+
+                QString str = criteria2->getSearchCriteria();
+                if( str.contains( "statistics." ) && !joins.contains( "statistics" ) )
+                    joins += " LEFT JOIN statistics ON statistics.url=tags.url AND statistics.deviceid=tags.deviceid";
+                if( str.contains( "devices." ) && !joins.contains( "devices" ) )
+                    joins += " LEFT JOIN devices ON tags.deviceid=devices.id";
+
+                if( i ) //multiple conditions
+                    str.prepend( " AND (");
+
+                criteriaListStr += str+")";
+            }
+            criteriaListStr += " )";
+        }
+
+        whereStr = " WHERE " + criteriaListStr;
     }
+   
+    //only select from mounted devices
+    whereStr = whereStr + " (*MountedDeviceSelection*) ";
 
     //order by expression
     if( m_orderCheck->isChecked() ) {
         if( m_orderCombo->currentItem() != m_orderCombo->count()-1 ) {
             QString field = m_dbFields[ m_orderCombo->currentItem() ];
-            QString table = field.left( field.find('.') );
-            if( !joins.contains( table ) ) {
-                if( table=="statistics")
-                    joins += " INNER JOIN statistics ON statistics.url=tags.url";
-            }
+            if( field.contains( "statistics." ) && !joins.contains( "statistics" ) )
+                joins += " LEFT JOIN statistics ON statistics.url=tags.url AND statistics.deviceid=tags.deviceid";
+
             QString orderType = m_orderTypeCombo->currentItem() == 1 ? " DESC" : " ASC";
             orderStr = " ORDER BY " +  field + orderType;
         }
@@ -357,7 +474,7 @@ void SmartPlaylistEditor::buildQuery()
             This is the score weighted random order.
             The RAND() function returns random values equally distributed between 0.0 (inclusive) and 1.0 (exclusive).
             The obvious way to get this order is to put every track <score> times into a list, sort the list by RAND()
-            (i.e. shuffle it) and discard every occurence of every track but the very first of each.
+            (i.e. shuffle it) and discard every occurrence of every track but the very first of each.
             By putting every track into the list only once but applying a transfer function
             T_s(x) := 1-(1-x)^(1/s) where s is the score, to RAND() before sorting the list, exactly the same
             distribution of tracks can be achieved (for a proof write to Stefan Siegel <kde@sdas.de>)
@@ -367,48 +484,31 @@ void SmartPlaylistEditor::buildQuery()
             */
            orderStr = " ORDER BY POWER(" + CollectionDB::instance()->randomFunc() + ",1.0/(statistics.percentage+1)) DESC";
             if( !joins.contains( "statistics" ) ) {
-                joins += " INNER JOIN statistics ON statistics.url=tags.url";
+                joins += " LEFT JOIN statistics ON statistics.url=tags.url";
             }
         }
     }
 
     if( m_limitCheck->isChecked() )
-        limitStr = " LIMIT 0," + QString::number( m_limitSpin->value() );
+        limitStr = " LIMIT " + QString::number( m_limitSpin->value() )+" OFFSET 0 ";
 
-    // album / artist / genre / title / year / comment / track / bitrate / length / samplerate / path
-    m_query = "SELECT album.name, artist.name, genre.name, tags.title, year.name, "
-                    "tags.comment, tags.track, tags.bitrate, tags.length, tags.samplerate, tags.url"
-                    " FROM " + joins + whereStr + orderStr + limitStr + ";";
+
+    // album / artist / composer / genre / title / year / comment / track / bitrate / discnumber / length / samplerate / path / compilation / filetype / bpm
+    m_query = "SELECT (*ListOfFields*) FROM "
+              + joins + whereStr + orderStr + limitStr + ";";
 
     if( m_expandCheck->isChecked() ) { //We use "(*ExpandString*)" as a marker, if a artist/track/album has this bizarre name, it won't work.
         QString field = m_expandableDbFields[ m_expandCombo->currentItem() ];
         QString table = field.left( field.find('.') );
         if( !joins.contains( table ) ) {
-            joins += " INNER JOIN statistics ON statistics.url=tags.url";
+            joins += " LEFT JOIN statistics ON statistics.url=tags.url AND statistics.deviceid=tags.deviceid";
         }
         if ( !criteriaListStr.isEmpty() )
             whereStr = QString(" WHERE (%1) AND %2 = '(*ExpandString*)'").arg(criteriaListStr).arg(field);
         else
-            whereStr = QString("WHERE %1 = '(*ExpandString*)'").arg(field);
-        m_expandQuery = "SELECT album.name, artist.name, genre.name, tags.title, year.name, "
-                            "tags.comment, tags.track, tags.bitrate, tags.length, tags.samplerate, tags.url"
-                            " FROM " + joins + whereStr + orderStr + limitStr + ";";
-    }
-}
-
-
-void SmartPlaylistEditor::updateMatchWidgets()
-{
-    int count = m_criteriaEditorList.count();
-    if( count > 1 ) {
-        m_matchCheck->setText( i18n("Match") );
-        m_matchCombo->show();
-        m_matchLabel->show();
-    }
-    else {
-        m_matchCheck->setText( i18n("Match the following conditions" ) );
-        m_matchCombo->hide();
-        m_matchLabel->hide();
+            whereStr = QString(" WHERE %1 = '(*ExpandString*)'").arg(field);
+        m_expandQuery = "SELECT (*ListOfFields*) FROM "
+                        + joins + whereStr + orderStr + limitStr + ";";
     }
 }
 
@@ -417,7 +517,7 @@ void SmartPlaylistEditor::updateMatchWidgets()
 //    CLASS CriteriaEditor
 ////////////////////////////////////////////////////////////////////////////
 
-CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, QDomElement criteria )
+CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, int criteriaType, QDomElement criteria )
     : QHBox( parent )
     , m_playlistEditor( editor )
     , m_currentValueType( -1 )
@@ -442,8 +542,14 @@ CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, QD
 
     connect( m_fieldCombo,    SIGNAL( activated(int) ), SLOT( slotFieldSelected(int) ) );
     connect( m_criteriaCombo, SIGNAL( activated(int) ), SLOT( loadEditWidgets() ) );
-    connect( m_addButton, SIGNAL( clicked() ), editor, SLOT( addCriteria() ) );
-    connect( m_removeButton, SIGNAL( clicked() ), SLOT( slotRemoveCriteria() ) );
+    if (criteriaType == SmartPlaylistEditor::criteriaAny) {
+	connect( m_addButton, SIGNAL( clicked() ), editor, SLOT( addCriteriaAny() ) );
+	connect( m_removeButton, SIGNAL( clicked() ), SLOT( slotRemoveCriteriaAny() ) );
+    }
+    else {
+	connect( m_addButton, SIGNAL( clicked() ), editor, SLOT( addCriteriaAll() ) );
+	connect( m_removeButton, SIGNAL( clicked() ), SLOT( slotRemoveCriteriaAll() ) );
+    }
 
     if ( !criteria.isNull() ) {
         int field = m_dbFields.findIndex( criteria.attribute( "field" ) );
@@ -481,6 +587,13 @@ CriteriaEditor::CriteriaEditor( SmartPlaylistEditor *editor, QWidget *parent, QD
                     m_intSpinBox2->setValue( values.last().toInt() );
                 break;
             }
+            case Rating:
+            {
+                m_comboBox->setCurrentItem( ratingToIndex( values.first().toInt() ) );
+                if( condition == i18n("is between") )
+                    m_comboBox2->setCurrentItem( ratingToIndex( values.last().toInt() ) );
+                break;
+            }
             case Date:
             {
                 if( condition == i18n("is in the last") || condition == i18n("is not in the last") ) {
@@ -535,10 +648,19 @@ QDomElement CriteriaEditor::getDomSearchCriteria( QDomDocument &doc )
             break;
          case Year: // fall through
          case Number:
+         {
             values << QString::number( m_intSpinBox1->value() );
             if( condition == i18n("is between")  )
                 values << QString::number( m_intSpinBox2->value() );
             break;
+         }
+         case Rating:
+         {
+            values << QString::number( indexToRating( m_comboBox->currentItem() ) );
+            if( condition == i18n("is between")  )
+                    values << QString::number( indexToRating( m_comboBox2->currentItem() ) );
+            break;
+         }
          case Date:
          {
             if( condition == i18n("is in the last") || condition == i18n("is not in the last") ) {
@@ -575,7 +697,10 @@ QString CriteriaEditor::getSearchCriteria()
     if( field.isEmpty() )
         return QString::null;
 
-    searchCriteria += field;
+    if ( ( field=="statistics.playcounter" || field=="statistics.rating" || field=="statistics.percentage" || field=="statistics.accessdate" || field=="statistics.createdate") )
+        searchCriteria += "COALESCE(" + field + ",0)";
+    else
+        searchCriteria += field;
 
     QString value;
     switch( getValueType( m_fieldCombo->currentItem() ) ) {
@@ -589,6 +714,13 @@ QString CriteriaEditor::getSearchCriteria()
             if( criteria == i18n("is between")  )
                 value += " AND " + QString::number( m_intSpinBox2->value() );
             break;
+        case Rating:
+        {
+            value = QString::number( indexToRating( m_comboBox->currentItem() ) );
+            if( criteria == i18n("is between")  )
+                value += " AND " + QString::number( indexToRating( m_comboBox2->currentItem() ) );
+            break;
+        }
         case Date:
         {
             if( criteria == i18n("is in the last") || criteria == i18n("is not in the last") ) {
@@ -618,9 +750,9 @@ QString CriteriaEditor::getSearchCriteria()
 
 
     if( criteria == i18n("contains") )
-        searchCriteria += " LIKE '%" + value + "%'";
+        searchCriteria += CollectionDB::likeCondition( value, true, true );
     else if( criteria == i18n("does not contain") )
-        searchCriteria += " NOT LIKE '%" + value + "%'";
+        searchCriteria += " NOT " + CollectionDB::likeCondition( value, true, true );
     else if( criteria == i18n("is") ) {
         if( m_currentValueType == Date )
             searchCriteria += " BETWEEN ";
@@ -629,8 +761,6 @@ QString CriteriaEditor::getSearchCriteria()
         if( m_currentValueType == String || m_currentValueType == AutoCompletionString )
             value.prepend("'").append("'");
         searchCriteria += value;
-        if ( ( field=="statistics.playcounter" || field=="statistics.percentage" ) && value=="0")
-            searchCriteria += " OR " + field + " IS NULL";
     }
     else if( criteria == i18n("is not") ) {
         if( m_currentValueType == Date )
@@ -642,9 +772,18 @@ QString CriteriaEditor::getSearchCriteria()
         searchCriteria += value;
     }
     else if( criteria == i18n("starts with") )
-        searchCriteria += " LIKE '" + value + "%'";
+    {
+        if( field == "tags.url" )
+        {
+            if( value.startsWith( "/" ) )
+                value = "." + value;
+            if( !value.startsWith( "./" ) )
+                value = "./" + value;
+        }
+        searchCriteria += CollectionDB::likeCondition( value, false, true );
+    }
     else if( criteria == i18n("ends with") )
-        searchCriteria += " LIKE '%" + value + "'";
+        searchCriteria += CollectionDB::likeCondition( value, true, false );
     else if( criteria == i18n("is greater than") || criteria == i18n("is after") )
         searchCriteria += " > " + value;
     else if( criteria == i18n("is smaller than") || criteria == i18n("is before" ) )
@@ -670,11 +809,25 @@ void CriteriaEditor::enableRemove( bool enable )
 }
 
 
-void CriteriaEditor::slotRemoveCriteria()
+void CriteriaEditor::slotRemoveCriteriaAny()
 {
-    m_playlistEditor->removeCriteria( this );
+    m_playlistEditor->removeCriteriaAny( this );
 }
 
+void CriteriaEditor::slotRemoveCriteriaAll()
+{
+    m_playlistEditor->removeCriteriaAll( this );
+}
+
+void CriteriaEditor::slotAddCriteriaAny()
+{
+    m_playlistEditor->addCriteriaAny();
+}
+
+void CriteriaEditor::slotAddCriteriaAll()
+{
+    m_playlistEditor->addCriteriaAll();
+}
 
 void CriteriaEditor::slotFieldSelected( int field )
 {
@@ -684,18 +837,20 @@ void CriteriaEditor::slotFieldSelected( int field )
     m_currentValueType = valueType;
 
     //enable auto-completion for artist, album and genre
-    if( valueType == AutoCompletionString ) { //Artist, Album, Genre
+    if( valueType == AutoCompletionString ) { //Artist, Composer, Album, Genre
         QStringList items;
         m_comboBox->clear();
         m_comboBox->completionObject()->clear();
 
         int currentField = m_fieldCombo->currentItem();
-        if( currentField == 0 ) //artist
+        if( currentField == FArtist ) //artist
            items = CollectionDB::instance()->artistList();
-        else if( currentField == 1 ) //album
+        else if( currentField == FComposer ) //composer
+           items = CollectionDB::instance()->composerList();
+        else if( currentField == FAlbum ) //album
            items = CollectionDB::instance()->albumList();
         else  //genre
-           items = MetaBundle::genreList();
+           items = CollectionDB::instance()->genreList();
 
         m_comboBox->insertStringList( items );
         m_comboBox->completionObject()->insertItems( items );
@@ -724,7 +879,7 @@ void CriteriaEditor::loadEditWidgets()
 
     QObjectList* list = m_editBox->queryList( "QWidget" );
     for( QObject *obj = list->first(); obj; obj = list->next()  )
-        ((QWidget*)obj)->deleteLater();
+        static_cast<QWidget*>(obj)->deleteLater();
 
     delete list;
 
@@ -738,11 +893,12 @@ void CriteriaEditor::loadEditWidgets()
             break;
         }
 
-        case AutoCompletionString:    //artist, album, genre
+        case AutoCompletionString:    //artist, composer, album, genre
         {
             m_comboBox = new KComboBox( true, m_editBox );
-            m_lineEdit = (KLineEdit*)m_comboBox->lineEdit();
+            m_lineEdit = static_cast<KLineEdit*>( m_comboBox->lineEdit() );
             m_lineEdit->setFocus();
+            m_comboBox->setMinimumSize( QSize( 240, 20 ) );
             m_comboBox->show();
             break;
         }
@@ -777,6 +933,24 @@ void CriteriaEditor::loadEditWidgets()
             break;
         }
 
+        case Rating:
+        {
+            const QStringList list = MetaBundle::ratingList();
+            m_comboBox = new KComboBox( false, m_editBox );
+            m_comboBox->insertStringList( list );
+            m_comboBox->show();
+
+            if( m_criteriaCombo->currentText() == i18n("is between") ) {
+                m_rangeLabel = new QLabel( i18n("and"), m_editBox );
+                m_rangeLabel->setAlignment( AlignCenter );
+                m_rangeLabel->show();
+                m_comboBox2 = new KComboBox( false, m_editBox );
+                m_comboBox2->insertStringList( list );
+                m_comboBox2->show();
+            }
+            break;
+        }
+
         case Date:
         {
             if( m_criteriaCombo->currentText() == i18n("is in the last") ||
@@ -814,7 +988,7 @@ void CriteriaEditor::loadEditWidgets()
 
 void CriteriaEditor::loadCriteriaList( int valueType, QString condition )
 {
-    if( m_currentValueType == valueType && condition == QString::null )
+    if( m_currentValueType == valueType && condition.isNull() )
         return;
 
     QStringList items;
@@ -826,6 +1000,7 @@ void CriteriaEditor::loadCriteriaList( int valueType, QString condition )
                   << i18n( "starts with" ) << i18n( "ends with" );
             break;
 
+        case Rating:
         case Number:
             items << i18n( "is" ) << i18n( "is not" ) << i18n( "is greater than" ) << i18n( "is smaller than" )
                   << i18n( "is between" );
@@ -855,23 +1030,30 @@ int CriteriaEditor::getValueType( int index )
     int valueType;
 
     switch( index ) {
-        case 0:
-        case 1:
-        case 2:
+        case FArtist:
+        case FComposer:
+        case FAlbum:
+        case FGenre:
             valueType = AutoCompletionString;
             break;
-        case 3:
-        case 7:
-        case 13:
+        case FTitle:
+        case FComment:
+        case FFilePath:
+        case FMountPoint:
             valueType = String;
             break;
-        case 4:
-        case 5:
-        case 8:
-        case 9:
+        case FLength:
+        case FTrack:
+        case FScore:
+        case FPlayCounter:
+        case FBPM:
+        case FBitRate:
             valueType = Number;
             break;
-        case 6:
+        case FRating:
+            valueType = Rating;
+            break;
+        case FYear:
             valueType = Year;
             break;
         default: valueType = Date;
