@@ -40,7 +40,7 @@ main( int argc, char** argv )
         }
         #endif
 
-        const char *plugin = NULL;
+        const char *plugin = 0;
 
         while( (plugin = visual_actor_get_next_by_name( plugin )) )
             std::cout << plugin << '\n';
@@ -87,7 +87,7 @@ main( int argc, char** argv )
         FD_ZERO( &fds );
         FD_SET( sockfd, &fds );
 
-        ::select( sockfd+1, &fds, NULL, NULL, &tv );
+        ::select( sockfd+1, &fds, 0, 0, &tv );
 
         if( FD_ISSET( sockfd, &fds) ) {
             //amaroK sent us some data
@@ -101,7 +101,7 @@ main( int argc, char** argv )
 
         //request pcm data
         ::send( sockfd, "PCM", 4, 0 );
-        nbytes = ::recv( sockfd, Vis::pcm_data, 512 * sizeof( int16_t ), 0 );
+        nbytes = ::recv( sockfd, Vis::pcm_data, 1024 * sizeof( int16_t ), 0 );
 
         render_time = LibVisual::render();
     }
@@ -253,7 +253,7 @@ namespace SDL
                       visual_bin_switch_actor_by_name( Vis::bin, (char*)Vis::plugin );
                     SDL::unlock();
 
-                    SDL_WM_SetCaption( Vis::plugin, NULL );
+                    SDL_WM_SetCaption( Vis::plugin, 0 );
 
                     break;
 
@@ -301,11 +301,11 @@ namespace LibVisual
     static int
     upload_callback( VisInput*, VisAudio *audio, void* )
     {
-        for( uint i = 0; i < 512; i++ )
-        {
-            audio->plugpcm[0][i] = pcm_data[i];
-            audio->plugpcm[1][i] = pcm_data[i];
-        }
+        VisBuffer buf;
+
+        visual_buffer_init( &buf, pcm_data, 1024, 0 );
+        visual_audio_samplepool_input( audio->samplepool, &buf, VISUAL_AUDIO_SAMPLE_RATE_44100,
+            VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_SAMPLE_CHANNEL_STEREO );
 
         return 0;
     }
@@ -330,7 +330,7 @@ namespace LibVisual
         bin    = visual_bin_new ();
         depth  = visual_video_depth_enum_from_value( 24 );
 
-        if( !plugin ) plugin = visual_actor_get_next_by_name( NULL );
+        if( !plugin ) plugin = visual_actor_get_next_by_name( 0 );
         if( !plugin ) exit( "Actor plugin not found!" );
 
         visual_bin_set_supported_depth( bin, VISUAL_VIDEO_DEPTH_ALL );
@@ -342,7 +342,7 @@ namespace LibVisual
 
         if( visual_bin_set_video( bin, video ) ) exit( "Cannot set video" );
 
-        visual_bin_connect_by_names( bin, (char*)plugin, NULL );
+        visual_bin_connect_by_names( bin, (char*)plugin, 0 );
 
         if( visual_bin_get_depth( bin ) == VISUAL_VIDEO_DEPTH_GL )
         {
